/*
 * Date (yymmdd):               090127
 * Version:                     0.1
 * Written by:                  Dr. Francisco J. Romero-Campero
 * Additional contributions by: Jonathan Blakes
 * (in no particular order)     Dr. Hongqing Cao
 *                              Dr. Natalio Krasnogor
 *				Dr. Jamie Twycross
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
 */

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "BNF.h"

using namespace ECSB;
using namespace boost::spirit::classic;
using namespace xmlpp;
using namespace std;


std::string ECSB::generateXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* psystemNode = xmlFile.create_root_node("PSystem");

    //Associated p system name
    std::string psystemName(tree->value.begin(), tree->value.end());
    psystemName = removeSpaces(psystemName);    
    psystemNode->set_attribute("name",psystemName);

    //Add children corresponding to the alphabet, membrane structure, initial multisets and rule sets
    xmlpp::Element* alphabetNode = psystemNode->add_child("alphabetOfObjects");
    xmlpp::Element* compartmentLabelsNode = psystemNode->add_child("compartmentLabels");
    xmlpp::Element* compartmentsNode = psystemNode->add_child("membraneStructure");
    xmlpp::Element* initialMultisetsNode = psystemNode->add_child("initialMultisets");
    xmlpp::Element* ruleSetsNode = psystemNode->add_child("ruleSets");

    for(iter_t child = tree->children.begin(); child != tree->children.end(); ++child)
    {
        std::string childName(child->value.begin(),child->value.end());

        if(childName == "alphabet")
        {
            //Adding the alphabet
            addAlphabet(alphabetNode,child);
        }
	else if(childName == "initialMultisets")
	{
	    //Adding the initial multisets
	    addInitialMultisets(initialMultisetsNode,child);
	}
        else if(childName == "compartments")
	{
            //Adding compartment labels
            addCompartmentLabels(compartmentLabelsNode,child);
            //Adding membrane structure
	    addMembraneStructure(compartmentsNode,child);
        }
        else if(childName == "ruleSets")
        {
            //Adding rule sets
            addRuleSets(ruleSetsNode,child);
        }
    }

    //Writing the XML file with a formatted style
    std::string fileName = psystemName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;

}

void ECSB::addAlphabet(xmlpp::Element* alphabetNode, iter_t alphabet)
{

    for(iter_t object = alphabet->children.begin(); object != alphabet->children.end(); ++object)
    {
        std::string objectName(object->value.begin(),object->value.end());
        objectName = removeSpaces(objectName);
        xmlpp::Element* objectNode = alphabetNode->add_child("object");
        objectNode->set_attribute("name",objectName);
    }

}

void ECSB::addInitialMultisets(xmlpp::Element* initialMultisetsNode, iter_t initialMultisets)
{

   for(iter_t compartmentName = initialMultisets->children.begin();
       compartmentName != initialMultisets->children.end();
       ++compartmentName)
   {
       //Create a node for each initial multiset
       std::string compName(compartmentName->value.begin(),compartmentName->value.end());
       compName = removeSpaces(compName);
       xmlpp::Element* initialMultisetNode = initialMultisetsNode->add_child("initialMultiset");
       initialMultisetNode->set_attribute("label",compName);

       //Adding the initial multiplicities for the objects in the corresponding initial multiset
       for(iter_t objectName = compartmentName->children.begin();
           objectName != compartmentName->children.end();
           ++objectName)
       {
           std::string objName(objectName->value.begin(), objectName->value.end());
           objName = removeSpaces(objName);
           iter_t multiplicity = objectName->children.begin();
	   std::string objMult(multiplicity->value.begin(), multiplicity->value.end());
           objMult = removeSpaces(objMult);
           xmlpp::Element* objectNode = initialMultisetNode->add_child("object");
	   objectNode->set_attribute("name",objName);
           objectNode->set_attribute("multiplicity",objMult);
       }

   }

}


void ECSB::addCompartmentLabels(xmlpp::Element* compartmentLabelsNode, iter_t compartments)
{

   for(iter_t compartmentName = compartments->children.begin();
       compartmentName != compartments->children.end();
       ++compartmentName)
   {
       std::string compName(compartmentName->value.begin(),compartmentName->value.end());
       compName = removeSpaces(compName);
       xmlpp::Element* labelNode = compartmentLabelsNode->add_child("compartmentLabel");
       labelNode->set_attribute("label",compName);

   }

}

void ECSB::addMembraneStructure(xmlpp::Element* compartmentsNode, iter_t compartments)
{
   for(iter_t compartmentName = compartments->children.begin();
       compartmentName != compartments->children.end();
       ++compartmentName)
   {
       std::string compName(compartmentName->value.begin(),compartmentName->value.end());
       compName = removeSpaces(compName);
       xmlpp::Element* compartmentNode = compartmentsNode->add_child("compartment");
       compartmentNode->set_attribute("name",compName);
       compartmentNode->set_attribute("label",compName);

       if(!compartmentName->children.empty())
       {
           iter_t fatherCompartment = compartmentName->children.begin();
	   std::string fatherName(fatherCompartment->value.begin(),fatherCompartment->value.end());
      fatherName = removeSpaces(fatherName);
	   compartmentNode->set_attribute("father",fatherName);
       }
   }
}

void ECSB::addRuleSets(xmlpp::Element* ruleSetsNode, iter_t ruleSets)
{

   for(iter_t compartmentName = ruleSets->children.begin();
       compartmentName != ruleSets->children.end();
       ++compartmentName)
   {
       std::string compName(compartmentName->value.begin(),compartmentName->value.end());
       compName = removeSpaces(compName);
       xmlpp::Element* ruleSetNode = ruleSetsNode->add_child("ruleSet");
       ruleSetNode->set_attribute("label",compName);

       for(iter_t ruleMod = compartmentName->children.begin();
           ruleMod != compartmentName->children.end();
           ++ruleMod)
       {
           //We distinguish between module and rule using the number of children in the corresponding node
           if(ruleMod->children.size() == 4)
           {
               addModule(ruleMod,ruleSetNode);
           }
           else if(ruleMod->children.size() == 7)
           {
               addRule(ruleMod,ruleSetNode);
           }
           else if(ruleMod->children.size() == 9)
           {
               addTranslRule(ruleMod,ruleSetNode);
           }

        }
   }
}

void ECSB::addModule(iter_t ruleMod, xmlpp::Element* ruleSetNode)
{
   //Name of the module	and addition of the corresponding node
   std::string moduleName(ruleMod->value.begin(), ruleMod->value.end());
   moduleName = removeSpaces(moduleName);
   xmlpp::Element* moduleNode = ruleSetNode->add_child("module");
   moduleNode->set_attribute("name",moduleName);

   //Details about the module
   iter_t objects = ruleMod->children.begin();
   iter_t constants = ruleMod->children.begin() + 1;
   iter_t compartments = ruleMod->children.begin() + 2;
   iter_t library = ruleMod->children.begin() + 3;

   //Adding the objects
   std::string objectsString;

   for(iter_t obj = objects->children.begin();
       obj != objects->children.end()-1;
       ++obj)
   {
       std::string objectName(obj->value.begin(),obj->value.end());
       objectsString += objectName + ",";
   }

   //Last objectName without ,
   iter_t lastObjectPointer = objects->children.end()-1;
   std::string lastObject(lastObjectPointer->value.begin(),lastObjectPointer->value.end());
   objectsString += lastObject;

   moduleNode->set_attribute("objects",objectsString);

   //Adding the constants
   std::string constantsString;

   for(iter_t cons = constants->children.begin();
       cons != constants->children.end()-1;
       ++cons)
   {
       std::string constantName(cons->value.begin(),cons->value.end());
       constantsString += constantName + ",";
   }

   //Last constantName without ,
   iter_t lastConstantPointer = constants->children.end()-1;
   std::string lastConstant(lastConstantPointer->value.begin(),lastConstantPointer->value.end());
   constantsString += lastConstant;

   moduleNode->set_attribute("constants",constantsString);

   //Adding the compartments
   std::string compartmentsString;

   for(iter_t comp = compartments->children.begin();
       comp != compartments->children.end()-1;
       ++comp)
   {
       std::string compartmentName(comp->value.begin(),comp->value.end());
       compartmentsString += compartmentName + ",";
   }

   //Last compartmentName without ,
   iter_t lastCompartmentPointer = compartments->children.end()-1;
   std::string lastCompartment(lastCompartmentPointer->value.begin(),lastCompartmentPointer->value.end());
   compartmentsString += lastCompartment;

   moduleNode->set_attribute("labels",compartmentsString);


   //Adding the library file
   std::string libraryName(library->value.begin(),library->value.end());
   libraryName = removeSpaces(libraryName);

   moduleNode->set_attribute("file",libraryName);

}

void ECSB::addRule(iter_t ruleMod, xmlpp::Element* ruleSetNode)
{

    //Name of the rule and addition of the corresponding node
    std::string ruleName(ruleMod->value.begin(), ruleMod->value.end());
    ruleName = removeSpaces(ruleName);
    xmlpp::Element* ruleNode = ruleSetNode->add_child("rule");
    ruleNode->set_attribute("name",ruleName);

    //LHS
    iter_t lhs = ruleMod->children.begin() + 1;
    //LHS outside
    iter_t lhsOutside = lhs->children.begin();
    //LHS inside
    iter_t lhsInside = lhs->children.begin() + 1;
    //LHS label
    iter_t lhsLabel = lhs->children.begin() + 2;

    //Constant Name
    iter_t constantNameDef = ruleMod->children.begin() + 3;
    std::string constantName(constantNameDef->value.begin(),constantNameDef->value.end());
    constantName = removeSpaces(constantName);
    ruleNode->set_attribute("constant",constantName);

    //RHS
    iter_t rhs = ruleMod->children.begin() + 5;
    //RHS outside
    iter_t rhsOutside = rhs->children.begin();
    //RHS inside
    iter_t rhsInside = rhs->children.begin() + 1;
    //RHS label
    iter_t rhsLabel = rhs->children.begin() + 2;

    //Constant
    if(ruleMod->children.size() == 7)
    {
        iter_t constant = ruleMod->children.begin() + 6;

        //Adding information about the stochastic constant
        iter_t constantValueDef = constant->children.begin() + 1;
        std::string constantValue(constantValueDef->value.begin(),constantValueDef->value.end());
        constantValue = removeSpaces(constantValue);

        ruleNode->set_attribute("value",constantValue);
    }

    //Adding information about the LHS
    xmlpp::Element* lhsNode = ruleNode->add_child("lhs");

    //LHS outside
    if(!lhsOutside->children.empty())
    {

        iter_t object = lhsOutside->children.begin();
        std::string objectName(object->value.begin(),object->value.end());
        objectName = removeSpaces(objectName);

        // Only one object can be in the outside on the lhs
        if(objectName == "+")
        {
            cerr << "error: rule " << ruleName << " has more than one object in the lhs outside";
            exit(EXIT_FAILURE);
        }

        xmlpp::Element* lhsOutsideObjects = lhsNode->add_child("listOfOutsideObjects");
        xmlpp::Element* objectNode = lhsOutsideObjects->add_child("object");
        objectNode->set_attribute("name",objectName);
    }

    //LHS inside
    if(!lhsInside->children.empty())
    {
        //Inside objects node
        xmlpp::Element* lhsInsideObjects = lhsNode->add_child("listOfInsideObjects");

        //Label
        std::string lhsLabelName(lhsLabel->value.begin(),lhsLabel->value.end());
        lhsLabelName = removeSpaces(lhsLabelName);
        lhsInsideObjects->set_attribute("label",lhsLabelName);

        //We distinguish if we have only one or two reactants by checking if there is a + separator
        iter_t check = lhsInside->children.begin();
        std::string checkValue(check->value.begin(),check->value.end());
        checkValue = removeSpaces(checkValue);

        if(checkValue == "+")
        {
            iter_t firstObject = check->children.begin();
            iter_t secondObject = firstObject + 1;

            std::string firstObjectName(firstObject->value.begin(),firstObject->value.end());
            firstObjectName = removeSpaces(firstObjectName);
            std::string secondObjectName(secondObject->value.begin(),secondObject->value.end());
            secondObjectName = removeSpaces(secondObjectName);

            // We only support rules with two reactants
            if(secondObjectName=="+")
            {
                cerr << "error: rule " << ruleName << " has more than two reactants.";
                exit(EXIT_FAILURE);
            }

            xmlpp::Element* firstObjectNode = lhsInsideObjects->add_child("object");
            firstObjectNode->set_attribute("name",firstObjectName);
            xmlpp::Element* secondObjectNode = lhsInsideObjects->add_child("object");
            secondObjectNode->set_attribute("name",secondObjectName);
        }
        else
        {
            //There is a single object

            //Label
            std::string lhsLabelName(lhsLabel->value.begin(),lhsLabel->value.end());
            lhsLabelName = removeSpaces(lhsLabelName);
            lhsInsideObjects->set_attribute("label",lhsLabelName);

            xmlpp::Element* objectNode = lhsInsideObjects->add_child("object");
            objectNode->set_attribute("name",checkValue);
        }
    }


    //Adding information about the RHS
    xmlpp::Element* rhsNode = ruleNode->add_child("rhs");

    //RHS outside
    if(!rhsOutside->children.empty())
    {
        iter_t object = rhsOutside->children.begin();
        std::string objectName(object->value.begin(),object->value.end());
        objectName = removeSpaces(objectName);

        // Only one object can be in the outside on the lhs
        if(objectName == "+")
        {
            cerr << "error: rule " << ruleName << " has more than one object in the lhs outside";
            exit(EXIT_FAILURE);
        }

        xmlpp::Element* rhsOutsideObjects = rhsNode->add_child("listOfOutsideObjects");
        xmlpp::Element* objectNode = rhsOutsideObjects->add_child("object");
        objectNode->set_attribute("name",objectName);
    }


    //RHS inside
    if(!rhsInside->children.empty())
    {
        //Inside objects node
        xmlpp::Element* rhsInsideObjects = rhsNode->add_child("listOfInsideObjects");

        //Label
        std::string rhsLabelName(rhsLabel->value.begin(),rhsLabel->value.end());
        rhsLabelName = removeSpaces(rhsLabelName);
        rhsInsideObjects->set_attribute("label",rhsLabelName);

        iter_t check = rhsInside->children.begin();
        std::string checkValue(check->value.begin(),check->value.end());
        checkValue = removeSpaces(checkValue);

        //We distinguish if we have only one or two reactants by checking if there is a + separator
        if(checkValue == "+")
        {
            iter_t firstObject = check->children.begin();
            iter_t secondObject = firstObject + 1;

            std::string firstObjectName(firstObject->value.begin(),firstObject->value.end());
            firstObjectName = removeSpaces(firstObjectName);
            std::string secondObjectName(secondObject->value.begin(),secondObject->value.end());
            secondObjectName = removeSpaces(secondObjectName);

            // We only support rules with two reactants
            if(secondObjectName=="+")
            {
                cerr << "error: rule " << ruleName << " has more than two products.";
                exit(EXIT_FAILURE);
            }

            xmlpp::Element* firstObjectNode = rhsInsideObjects->add_child("object");
            firstObjectNode->set_attribute("name",firstObjectName);
            xmlpp::Element* secondObjectNode = rhsInsideObjects->add_child("object");
            secondObjectNode->set_attribute("name",secondObjectName);
        }
        else
        {
            //There is a single object

            //Label
            std::string rhsLabelName(rhsLabel->value.begin(),rhsLabel->value.end());
            rhsLabelName = removeSpaces(rhsLabelName);
            rhsInsideObjects->set_attribute("label",rhsLabelName);

            xmlpp::Element* objectNode = rhsInsideObjects->add_child("object");
            objectNode->set_attribute("name",checkValue);
        }
    }

}

std::string ECSB::generateLibraryXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* libraryNode = xmlFile.create_root_node("libraryOfModules");

    //Associated p system name
    std::string libraryName(tree->value.begin(), tree->value.end());
    libraryName = removeSpaces(libraryName);
    libraryNode->set_attribute("name",libraryName);

    for(iter_t child = tree->children.begin(); child != tree->children.end(); ++child)
    {
        std::string moduleName(child->value.begin(),child->value.end());
        moduleName = removeSpaces(moduleName);
        xmlpp::Element* moduleNode = libraryNode->add_child("moduleDefinition");
        moduleNode->set_attribute("name",moduleName);

        //The different attributes of a module (objects,constants and compartments) and the set of rules associated with it.
        iter_t objectsDef = child->children.begin();
        iter_t constantsDef = child->children.begin() + 1;
        iter_t compartmentsDef = child->children.begin() + 2;
        iter_t ruleSetDef = child->children.begin() + 3;

        //Creating the corresponding xml nodes for the attributes and set of rules.
        xmlpp::Element* objectsNode = moduleNode->add_child("setOfObjects");
        xmlpp::Element* constantsNode = moduleNode->add_child("setOfConstants");
        xmlpp::Element* compartmentsNode = moduleNode->add_child("setOfLabels");
        xmlpp::Element* rulesNode = moduleNode->add_child("setOfRules");

        //Objects
        for(iter_t obj = objectsDef->children.begin(); obj != objectsDef->children.end(); ++obj)
        {
            std::string objectName(obj->value.begin(),obj->value.end());
            objectName = removeSpaces(objectName);
            xmlpp::Element* objectNode = objectsNode->add_child("object");
            objectNode->set_attribute("name",objectName);
        }

        //Constants
        for(iter_t cons = constantsDef->children.begin(); cons != constantsDef->children.end(); ++cons)
        {
            std::string constantName(cons->value.begin(),cons->value.end());
            constantName = removeSpaces(constantName);
            xmlpp::Element* constantNode = constantsNode->add_child("constant");
            constantNode->set_attribute("name",constantName);
        }

        //Compartments
        for(iter_t comp = compartmentsDef->children.begin(); comp != compartmentsDef->children.end(); ++comp)
        {
            std::string compartmentName(comp->value.begin(),comp->value.end());
            compartmentName = removeSpaces(compartmentName);
            xmlpp::Element* compartmentNode = compartmentsNode->add_child("label");
            compartmentNode->set_attribute("name",compartmentName);
        }

        //Rules and modules
        for(iter_t r = ruleSetDef->children.begin(); r != ruleSetDef->children.end(); ++r)
        {
            int numberChildren = r->children.size();

            std::string ruleName(r->value.begin(),r->value.end());
            ruleName = removeSpaces(ruleName);
            
            if(numberChildren == 4)
            {
                addModule(r,rulesNode);
            }
            else if(numberChildren == 7 || numberChildren == 6)
            {
                addRule(r,rulesNode);
            }
            else if(numberChildren == 9 || numberChildren == 8)
            {
                addTranslRule(r,rulesNode);
            }
        }


    }

    //Writing the XML file with a formatted style
    std::string fileName = libraryName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;
    
}


void ECSB::addTranslRule(iter_t ruleMod, xmlpp::Element* ruleSetNode)
{

    //Name of the rule and addition of the corresponding node
    std::string ruleName(ruleMod->value.begin(), ruleMod->value.end());
    ruleName = removeSpaces(ruleName);
    xmlpp::Element* ruleNode = ruleSetNode->add_child("rule");
    ruleNode->set_attribute("name",ruleName);

    //LHS
    iter_t lhs = ruleMod->children.begin() + 1;
    //LHS objects
    iter_t lhsObjects = lhs->children.begin();
    //LHS label
    iter_t lhsLabel = lhs->children.begin() + 1;

    //rule vector
    iter_t ruleVectorDef = lhs->children.begin() + 2;

    //Constant Name
    iter_t constantNameDef = ruleMod->children.begin() + 5;
    std::string constantName(constantNameDef->value.begin(),constantNameDef->value.end());
    constantName = removeSpaces(constantName);
    ruleNode->set_attribute("constant",constantName);

    //RHS
    iter_t rhs = ruleMod->children.begin() + 7;
    //RHS objects
    iter_t rhsObjects = rhs->children.begin() + 2;

    //Constant
    if(ruleMod->children.size() == 9)
    {
        iter_t constant = ruleMod->children.begin() + 8;

        //Adding information about the stochastic constant
        iter_t constantValueDef = constant->children.begin() + 1;
        std::string constantValue(constantValueDef->value.begin(),constantValueDef->value.end());
        constantValue = removeSpaces(constantValue);

        ruleNode->set_attribute("value",constantValue);
    }

    //Adding information about the LHS
    xmlpp::Element* lhsNode = ruleNode->add_child("lhs");

    //Adding rule vector
    iter_t xDef = ruleVectorDef->children.begin();
    iter_t yDef = ruleVectorDef->children.begin() + 1;

    std::string xValue(xDef->value.begin(),xDef->value.end());
    xValue = removeSpaces(xValue);
    std::string yValue(yDef->value.begin(),yDef->value.end());
    yValue = removeSpaces(yValue);
    std::string vectorString = xValue + "," + yValue;

    ruleNode->set_attribute("vector",vectorString);
    
    //LHS objects
    if(!lhsObjects->children.empty())
    {
        xmlpp::Element* lhsListOfTranslocatedObjects = lhsNode->add_child("listOfTranslocatedObjects");

        //Label
        std::string lhsLabelName(lhsLabel->value.begin(),lhsLabel->value.end());
        lhsLabelName = removeSpaces(lhsLabelName);        

        lhsListOfTranslocatedObjects->set_attribute("label",lhsLabelName);

        //Objects
        iter_t object = lhsObjects->children.begin();
        std::string objectName(object->value.begin(),object->value.end());
        objectName = removeSpaces(objectName);

        // Only one object can be translocated
        if(objectName == "+")
        {
            cerr << "error: rule " << ruleName << " has more than one object to translocated";
            exit(EXIT_FAILURE);
        }

        xmlpp::Element* objectNode = lhsListOfTranslocatedObjects->add_child("object");
        objectNode->set_attribute("name",objectName);
    }
    else
    {
        cerr << "No objects in the LHS of rule " << ruleName << ". No objects to translocate." ;
        exit(EXIT_FAILURE);
    }

    //Adding information about the RHS
    xmlpp::Element* rhsNode = ruleNode->add_child("rhs");

    //RHS objects
    if(!rhsObjects->children.empty())
    {
        xmlpp::Element* rhsListOfTranslocatedObjects = rhsNode->add_child("listOfTranslocatedObjects");

        //Label
        std::string lhsLabelName(lhsLabel->value.begin(),lhsLabel->value.end());
        lhsLabelName = removeSpaces(lhsLabelName);
        rhsListOfTranslocatedObjects->set_attribute("label",lhsLabelName);

        //Objects
        iter_t object = rhsObjects->children.begin();
        std::string objectName(object->value.begin(),object->value.end());
        objectName = removeSpaces(objectName);

        // Only one object can be translocated
        if(objectName == "+")
        {
            cerr << "error: rule " << ruleName << " has more than one object to translocated";
            exit(EXIT_FAILURE);
        }

        xmlpp::Element* objectNode = rhsListOfTranslocatedObjects->add_child("object");
        objectNode->set_attribute("name",objectName);
    }
    else
    {
        cerr << "No objects in the RHS of rule " << ruleName << ". No objects to translocate." ;
        exit(EXIT_FAILURE);
    }

}

std::string ECSB::generateLatticeXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* latticeNode = xmlFile.create_root_node("lattice");

    //Associated lattice name
    std::string latticeName(tree->value.begin(), tree->value.end());
    latticeName = removeSpaces(latticeName);
    latticeNode->set_attribute("name",latticeName);

    //Add children corresponding to the alphabet, membrane structure, initial multisets and rule sets
    //xmlpp::Element* alphabetNode = psystemNode->add_child("alphabetOfObjects");
    //xmlpp::Element* compartmentLabelsNode = psystemNode->add_child("compartmentLabels");
    //xmlpp::Element* compartmentsNode = psystemNode->add_child("membraneStructure");
    //xmlpp::Element* initialMultisetsNode = psystemNode->add_child("initialMultisets");
    //xmlpp::Element* ruleSetsNode = psystemNode->add_child("ruleSets");

    for(iter_t child = tree->children.begin(); child != tree->children.end(); ++child)
    {
        std::string childName(child->value.begin(),child->value.end());
        childName = removeSpaces(childName);

        if(childName == "dimension")
        {
            //Adding the dimension
            iter_t dimDef = child->children.begin();
            std::string dim(dimDef->value.begin(),dimDef->value.end());
				dim = removeSpaces(dim);
            latticeNode->set_attribute("dimension",dim);
        }
        else if(childName == "scale")
	{
            //Adding the scale
            iter_t scaleDef = child->children.begin();
            std::string scale(scaleDef->value.begin(),scaleDef->value.end());
            scale = removeSpaces(scale);
            latticeNode->set_attribute("scale",scale);
	}
        else if(childName == "units")
        {
            //Adding the units
            iter_t unitsDef = child->children.begin();
            std::string units(unitsDef->value.begin(),unitsDef->value.end());
            units = removeSpaces(units);
            latticeNode->set_attribute("units",units);
        }
        else if(childName == "xmin")
        {
            //Adding the units
            iter_t xminDef = child->children.begin();
            std::string xmin(xminDef->value.begin(),xminDef->value.end());
            xmin = removeSpaces(xmin);
            latticeNode->set_attribute("xmin",xmin);
        }
        else if(childName == "xmax")
        {
            //Adding the units
            iter_t xmaxDef = child->children.begin();
            std::string xmax(xmaxDef->value.begin(),xmaxDef->value.end());
            xmax = removeSpaces(xmax);
            latticeNode->set_attribute("xmax",xmax);
        }
        else if(childName == "ymin")
        {
            //Adding the units
            iter_t yminDef = child->children.begin();
            std::string ymin(yminDef->value.begin(),yminDef->value.end());
            ymin = removeSpaces(ymin);
            latticeNode->set_attribute("ymin",ymin);
        }
        else if(childName == "ymax")
        {
            //Adding the units
            iter_t ymaxDef = child->children.begin();
            std::string ymax(ymaxDef->value.begin(),ymaxDef->value.end());
            ymax = removeSpaces(ymax);
            latticeNode->set_attribute("ymax",ymax);
        }
        else if(childName == "parameters")
        {
            for(iter_t param = child->children.begin(); param != child->children.end(); ++param)
            {
                //Adding individual parameter
                std::string parameterName(param->value.begin(),param->value.end());
                parameterName = removeSpaces(parameterName);
                iter_t valueDef = param->children.begin();
                std::string valueStr(valueDef->value.begin(),valueDef->value.end());
                valueStr = removeSpaces(valueStr);
                xmlpp::Element* parameterNode = latticeNode->add_child("parameter");
                parameterNode->set_attribute("name",parameterName);
                parameterNode->set_attribute("value",valueStr);
            }
        }
        else if(childName == "basis")
        {
            xmlpp::Element* basisNode = latticeNode->add_child("basis");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = basisNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }
        else if(childName == "vertices")
        {
            xmlpp::Element* verticesNode = latticeNode->add_child("vertices");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = verticesNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }
        else if(childName == "neighbours")
        {
            xmlpp::Element* neighboursNode = latticeNode->add_child("neighbours");
            for(iter_t basis = child->children.begin(); basis != child->children.end(); ++basis)
            {
                //Adding individual vector
                xmlpp::Element* vectorNode = neighboursNode->add_child("vector");

                iter_t xComp = basis->children.begin();
                iter_t yComp = basis->children.begin() + 1;

                std::string xValue(xComp->value.begin(),xComp->value.end());
                xValue = removeSpaces(xValue);
                std::string yValue(yComp->value.begin(),yComp->value.end());
                yValue = removeSpaces(yValue);

                vectorNode->set_attribute("x",xValue);
                vectorNode->set_attribute("y",yValue);
            }
        }


    }

    //Writing the XML file with a formatted style
    std::string fileName = latticeName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;
}

std::string ECSB::generateLPPXML(iter_t tree)
{
    //Create xml file and root node
    xmlpp::Document xmlFile;
    xmlpp::Element* LPPSystemNode = xmlFile.create_root_node("CAPSystemModel");

    //Associated lattice name
    std::string LPPSystemName(tree->value.begin(), tree->value.end());
    LPPSystemName = removeSpaces(LPPSystemName);
    LPPSystemNode->set_attribute("name",LPPSystemName);

	 //std::cout << "name:" << LPPSystemName << std::endl;

    //Adding the SPsystems
    addSPsystems(tree->children.begin(),LPPSystemNode);

	 //std::cout << "SP systems added" << std::endl;

    //Adding the lattice
    addLattice(tree->children.begin()+1,LPPSystemNode);

	 //std::cout << "Lattice added" << std::endl;

    //Adding the spatial distribution
    addSpatialDistribution(tree->children.begin()+2,LPPSystemNode);

	 //std::cout << "spatial distribution added" << std::endl;

    //Writing the XML file with a formatted style
    std::string fileName = LPPSystemName + ".xml";
    xmlFile.write_to_file_formatted(fileName);

    return fileName;

}

void ECSB::addSPsystems(iter_t SPsystemsDeclaration, xmlpp::Node* LPPSystemNode)
{
    //Adding node containing the list of SP systems
    xmlpp::Element* SPsystemsNode = LPPSystemNode->add_child("listOfPSystems");

    //Adding a node for each SP system including its name and file
    for(iter_t SPsyst = SPsystemsDeclaration->children.begin();
        SPsyst != SPsystemsDeclaration->children.end();
        ++SPsyst)
    {
        std::string SPsystemName(SPsyst->value.begin(),SPsyst->value.end());
        SPsystemName = removeSpaces(SPsystemName);
        iter_t fileDeclaration = SPsyst->children.begin();
        std::string SPsystemFile(fileDeclaration->value.begin(),fileDeclaration->value.end());
        SPsystemFile = removeSpaces(SPsystemFile);
        xmlpp::Element* SPsystemNode = SPsystemsNode->add_child("PSystem");
        SPsystemNode->set_attribute("name",SPsystemName);
        SPsystemNode->set_attribute("file",SPsystemFile);

		  //std::cout << "SPSystem " << SPsystemName << " from " << SPsystemFile << std::endl; 

    }
}


void ECSB::addLattice(iter_t latticeDeclaration, xmlpp::Node* LPPSystemNode)
{
    //Adding node containg the lattice specification: its name and file
    xmlpp::Element* latticeNode = LPPSystemNode->add_child("lattice");

    //Adding attributes to the lattice: name and file
    std::string latticeName(latticeDeclaration->value.begin(),latticeDeclaration->value.end());
    latticeName = removeSpaces(latticeName);
    latticeNode->set_attribute("name",latticeName);

    iter_t latticeFileDef = latticeDeclaration->children.begin();
    std::string latticeFile(latticeFileDef->value.begin(),latticeFileDef->value.end());
    latticeFile = removeSpaces(latticeFile);
    latticeNode->set_attribute("file",latticeFile);

}

void ECSB::addSpatialDistribution(iter_t spatialDistributionDef, xmlpp::Node* LPPSystemNode)
{
    //Adding the list of positions
    xmlpp::Element* positionsNode = LPPSystemNode->add_child("cellularAutomata");

    //Adding each position
    for(iter_t pos = spatialDistributionDef->children.begin();
        pos != spatialDistributionDef->children.end();
        ++pos)
    {
        xmlpp::Element* positionNode = positionsNode->add_child("automaton");
        std::string SPsystemName(pos->value.begin(),pos->value.end());
        SPsystemName = removeSpaces(SPsystemName);
        positionNode->set_attribute("pSystem",SPsystemName);

        for(iter_t element = pos->children.begin();
            element != pos->children.end();
            ++element)
        {
            std::string elementName(element->value.begin(),element->value.end());
            elementName = removeSpaces(elementName);

            //Adding the parameters used in the positions
            if(elementName == "parameters")
            {
                addParameters(element,positionNode);
            }
            //Adding the coordinates for the positions
            else if(elementName == "coordinates")
            {
                addCoordinates(element,positionNode);
            }
        }

    }

}


void ECSB::addParameters(iter_t parametersDef, xmlpp::Node* positionNode)
{
    for(iter_t para = parametersDef->children.begin();
        para != parametersDef->children.end();
        ++para)
    {
        //Adding individual parameters
        xmlpp::Element* parameterNode = positionNode->add_child("parameter");
        std::string parameterName(para->value.begin(),para->value.end());
        parameterName = removeSpaces(parameterName);
        parameterNode->set_attribute("name",parameterName);

        //Getting lower bound, step and upperbound
        iter_t lowDef = para->children.begin();
        std::string lowStr(lowDef->value.begin(),lowDef->value.end());
        lowStr = removeSpaces(lowStr);
        parameterNode->set_attribute("lowerBound",lowStr);

        iter_t stepDef = para->children.begin() + 1;
        std::string stepStr(stepDef->value.begin(),stepDef->value.end());
        stepStr = removeSpaces(stepStr);
        parameterNode->set_attribute("step",stepStr);

        iter_t upDef = para->children.begin() + 2;
        std::string upStr(upDef->value.begin(),upDef->value.end());
        upStr = removeSpaces(upStr);
        parameterNode->set_attribute("upperBound",upStr);      
    }
}


void ECSB::addCoordinates(iter_t coordinatesDef, xmlpp::Node* positionNode)
{
    //Adding x coordinate
    iter_t xDef = (coordinatesDef->children.begin())->children.begin();
    std::string xStr(xDef->value.begin(),xDef->value.end());
    xStr = removeSpaces(xStr);
    xmlpp::Element* xNode = positionNode->add_child("Xcoordinate");
    xNode->set_attribute("value",xStr);

    //Adding y coordinate
    if(coordinatesDef->children.size() == 2)
    {
        iter_t yDef = (coordinatesDef->children.begin() + 1)->children.begin();
        std::string yStr(yDef->value.begin(),yDef->value.end());
        yStr = removeSpaces(yStr);
        xmlpp::Element* yNode = positionNode->add_child("Ycoordinate");
        yNode->set_attribute("value",yStr);

    }

}

std::string ECSB::removeSpaces(std::string str)
{

	//std::cout << "old string:" << str << std::endl;
   
   size_t pos = str.find(' ');

   while(pos != str.npos)
   {
       str.erase(pos,1);
	    pos = str.find(' ');
   }

   pos = str.find("\t");

   while(pos != str.npos)
   {
       str.erase(pos,1);
	    pos = str.find("\t");
   }

	//for(int j=0; j<str.length(); j++)
	//{
		//if(str[j]==' ')
		//str.erase(j,1);
	//}

	//std::cout << "new string:" << str << std::endl;

	return str;

}



