/*
 * Date (yymmdd):               090127
 * Version:                     0.1
 * Written by:                  Dr. Francisco J. Romero-Campero
 * Additional contributions by: Jonathan Blakes
 * (in no particular order)     Dr. Hongqing Cao
 *                              Dr. Natalio Krasnogor
 *				Dr. Jamie Twycross
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
 */

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "BNF.h"
#include "CAPSystem.h"
#include "Misc.h"
#include <list>
#include <vector>

using namespace ECSB;
using namespace std;
using namespace xmlpp;

///////////////////////////////
// lattice class definitions //
///////////////////////////////

//Default Constructor
lattice::lattice()
{

}

//Default Destructor
lattice::~lattice()
{

}

//Constructor
lattice::lattice(std::string latticeFileName) //xmlpp::Node* latticeDeclaration)
{
    //Variable for the xml file with the definition of a lattice
    std::string xmlFile;

    //According to the format of the specification the corresponding constructor
    //is invoked.
    size_t pos = latticeFileName.find('.');
    std::string fileExtension = latticeFileName.substr(pos+1);

    if(fileExtension == "lat")
    {
        lattice_p latticeParser;    //  Our parser

        //Generate a string with the content of the file
        std::string stringFile;
        std::string line;

        ifstream inputFile(latticeFileName.c_str());

        if (inputFile.is_open())
        {
            while (! inputFile.eof() )
            {
                getline (inputFile,line);
                //cout << line << endl;
                stringFile += line;
            }

            inputFile.close();
        }
        else
        {
            cerr << "Unable to open file 3" << std::endl;
            exit(EXIT_FAILURE);
        }

        //We parse the content of the file to generate a abstract syntax tree
        tree_parse_info<> result = ast_parse(stringFile.c_str(),latticeParser,space_p);

        if (result.full)
        {
            //cout << "-------------------------\n";
            //cout << "Parsing succeeded\n";
            //cout << "-------------------------\n";

            //When the parsing is succesfull the xml file is generated
            iter_t iter = result.trees.begin();
            xmlFile = generateLatticeXML(iter);
        }
        else
        {
                //Failed parsing
                cerr << "Parsing of file " << latticeFileName << " according to BNF failed at: " << std::endl;
                std::cout << result.stop << std::endl;
                exit(EXIT_FAILURE);
        }
    }
    //Specification in xml format
    else if(fileExtension == "xml")
    {
        xmlFile = latticeFileName;
    }


  //Use of the DOM parser
  DomParser parser;
  parser.parse_file(xmlFile);
  if(!parser)
  {
      cerr << "Parsing Unsuccesfull" << std::endl;
      exit(EXIT_FAILURE);
  }

  xmlpp::Node* latticeDeclaration = parser.get_document()->get_root_node();

  //Get the different attributes of the lattice
  xmlpp::Element* latticeElement = dynamic_cast<xmlpp::Element*>(latticeDeclaration);
  this->name = (latticeElement->get_attribute("name"))->get_value();
  this->dimension = fromStringTo<int>((latticeElement->get_attribute("dimension"))->get_value());

  xmlpp::Attribute* unitsDeclaration = latticeElement->get_attribute("units");
  if(unitsDeclaration != NULL)
  {
    this->units = unitsDeclaration->get_value();
  }

  xmlpp::Attribute* scaleDeclaration = latticeElement->get_attribute("scale");
  if(scaleDeclaration != NULL)
  {
      this->scale = fromStringTo<int>(scaleDeclaration->get_value());
  }

  xmlpp::Attribute* xminDeclaration =latticeElement->get_attribute("xmin");
  this->xmin = fromStringTo<int>(xminDeclaration->get_value());

  xmlpp::Attribute* xmaxDeclaration =latticeElement->get_attribute("xmax");
  this->xmax = fromStringTo<int>(xmaxDeclaration->get_value());

  xmlpp::Attribute* yminDeclaration =latticeElement->get_attribute("ymin");
  this->ymin = fromStringTo<int>(yminDeclaration->get_value());

  xmlpp::Attribute* ymaxDeclaration =latticeElement->get_attribute("ymax");
  this->ymax = fromStringTo<int>(ymaxDeclaration->get_value());

  xmlpp::Node::NodeList parametersDeclaration = latticeDeclaration->get_children("parameter");
  for(xmlpp::Node::NodeList::iterator par = parametersDeclaration.begin();
      par != parametersDeclaration.end();
      ++par)
    {
      xmlpp::Element* parElement = dynamic_cast<xmlpp::Element*>(*par);
      std::string parName = (parElement->get_attribute("name"))->get_value();
      double parValue = fromStringTo<double>((parElement->get_attribute("value"))->get_value());
      this->parameters.insert(pair<std::string,double>(parName,parValue));
    }

  //Basis declaration
  xmlpp::Node* basisDeclaration = (latticeDeclaration->get_children("basis")).front();
  xmlpp::Node::NodeList vectors = basisDeclaration->get_children("vector");

  for(xmlpp::Node::NodeList::iterator vect = vectors.begin();
      vect != vectors.end();
      ++vect)
    {
      xmlpp::Element* vectorElement = dynamic_cast<xmlpp::Element*>(*vect);

      xmlpp::Attribute* xDim = vectorElement->get_attribute("x");
      xmlpp::Attribute* yDim = vectorElement->get_attribute("y");

      std::vector<std::string> basisVector;
      if(xDim != NULL)
	basisVector.push_back(xDim->get_value());
      if(yDim != NULL)
	basisVector.push_back(yDim->get_value());

      (this->basis).push_back(basisVector);
    }

  //Neighbours declaration
  xmlpp::Node* neighboursDeclaration = (latticeDeclaration->get_children("neighbours")).front();
  xmlpp::Node::NodeList neighVectors = neighboursDeclaration->get_children("vector");

  for(xmlpp::Node::NodeList::iterator vect = neighVectors.begin();
      vect != neighVectors.end();
      ++vect)
    {
      xmlpp::Element* vectorElement = dynamic_cast<xmlpp::Element*>(*vect);

      xmlpp::Attribute* xDim = vectorElement->get_attribute("x");
      xmlpp::Attribute* yDim = vectorElement->get_attribute("y");

      std::vector<std::string> neighVector;
      if(xDim != NULL)
	neighVector.push_back(xDim->get_value());
      if(yDim != NULL)
	neighVector.push_back(yDim->get_value());

      (this->neighbours).push_back(neighVector);
    }

  //Vertices declaration
  xmlpp::Node::NodeList verticesDeclaration = latticeDeclaration->get_children("vertices");

  if(!verticesDeclaration.empty())
  {
      xmlpp::Node* vertices = verticesDeclaration.front();
      xmlpp::Node::NodeList verticesVectors = vertices->get_children("vector");


      for(xmlpp::Node::NodeList::iterator vect = verticesVectors.begin();
        vect != verticesVectors.end();
        ++vect)
        {
            xmlpp::Element* vectorElement = dynamic_cast<xmlpp::Element*>(*vect);

            xmlpp::Attribute* xDim = vectorElement->get_attribute("x");
            xmlpp::Attribute* yDim = vectorElement->get_attribute("y");

            std::vector<std::string> vertexVector;
            if(xDim != NULL)
                vertexVector.push_back(xDim->get_value());
            if(yDim != NULL)
                vertexVector.push_back(yDim->get_value());

            (this->vertices).push_back(vertexVector);
        }
  }

}

//Getters
int lattice::getXMax()
{
    return this->xmax;
}

int lattice::getYMax()
{
    return this->ymax;
}

void lattice::show()
{
  std::cout << "------------------------------" << std::endl;
  std::cout << "Lattice: " << this->name << std::endl;
  std::cout << "\tdimension:" << this->dimension << std::endl;
  std::cout << "\tunits: " << this->units << std::endl;
  std::cout << "\tscale: " << this->scale << std::endl;
  std::cout << "\txmax: " << this->xmax << std::endl;

  if(this->dimension >= 2)
    std::cout << "\tymax: " << this->ymax << std::endl;


  std::cout << "\tparameters:" << std::endl;
  for(std::map<std::string,double>::iterator par = (this->parameters).begin();
      par != (this->parameters).end();
      ++par)
    {
      std::cout << "\t\tparameter: " << par->first << " " << par->second << std::endl;
    }

  std::cout << "\tbasis:" << std::endl;
  for(std::vector< std::vector<string> >::iterator vect = (this->basis).begin();
      vect != (this->basis).end();
      ++vect)
    {
      std::cout << "\t\tvector: ";
      for(std::vector<std::string>::iterator c = vect->begin();
	  c != vect->end();
	  ++c)
	{
	  std::cout << *c << " ";
	}
      std::cout << std::endl;

    }

  std::cout << std::endl <<"\tneighbours:" << std::endl;
  for(std::vector< std::vector<string> >::iterator vect = (this->neighbours).begin();
      vect != (this->neighbours).end();
      ++vect)
    {
      std::cout << "\t\tvector: ";
      for(std::vector<std::string>::iterator c = vect->begin();
	  c != vect->end();
	  ++c)
	{
	  std::cout << *c << " ";
	}
      std::cout << std::endl;

    }

  std::cout << std::endl <<"\tvertices:" << std::endl;
  for(std::vector< std::vector<string> >::iterator vect = (this->vertices).begin();
      vect != (this->vertices).end();
      ++vect)
    {
      std::cout << "\t\tvector: ";
      for(std::vector<std::string>::iterator c = vect->begin();
	  c != vect->end();
	  ++c)
	{
	  std::cout << *c << " ";
	}
      std::cout << std::endl;

    }




  std::cout << "------------------------------" << std::endl;

}

/////////////////////////////////
// automaton class definitions //
/////////////////////////////////

//Default Constructor
automaton::automaton()
{

}

  //Copy constructor
automaton::automaton(const automaton& aut)
{
    this->multiset = aut.multiset;
    this->pSystem = aut.pSystem;
    this->position = aut.position;
}


//Constructor from coordinates and P system name
automaton::automaton(std::vector<int> coordinates, std::string pSystemName)
{
  this->position = coordinates;
  this->pSystem = pSystemName;
}

//Constructor from coordinates, P system name and initial multiset
automaton::automaton(std::vector<int> coordinates, std::string pSystemName, InitialMultiset initialMultiset)
{
  this->position = coordinates;
  this->pSystem = pSystemName;
  this->multiset = initialMultiset;
}

//Default Destructor
automaton::~automaton()
{

}

//Getters
std::vector<int> automaton::getPosition()
{
  return this->position;
}

std::string automaton::getPSystemName()
{
  return this->pSystem;
}

InitialMultiset automaton::getMultiset()
{
    return this->multiset;
}


//Setters
void automaton::setPosition(std::vector<int> pos)
{
    this->position = pos;
}

void automaton::setPSystemName(std::string pSystemName)
{
    this->pSystem = pSystemName;
}


//Displayer
void automaton::show()
{
  std::cout << "-----------------------" << std::endl;
  std::cout << "Cell Automaton: " << std::endl;
  std::cout << "\tPosition: ( ";
  for(std::vector<int>::iterator coord = (this->position).begin();
      coord != (this->position).end();
      ++coord)
    {
      std::cout << *coord << " ";
    }
  std::cout << ")" << std::endl;
  std::cout << "\tP System Name: " << this->pSystem << std:: endl;

}

/////////////////////////////////
// CAPsystem class definitions //
/////////////////////////////////

//Default Constructor
CAPSystem::CAPSystem()
{

}

  //Copy constructor
CAPSystem::CAPSystem(const CAPSystem& capSystem)
{
    this->cellAutomata = capSystem.cellAutomata;
    this->name = capSystem.name;
    this->pSystems = capSystem.pSystems;
    this->spatialLattice = capSystem.spatialLattice;
}


//Constructor from XML file
CAPSystem::CAPSystem(std::string fileName)
{

    //Variable for the xml file with the definition of a lattice
    std::string xmlFile;

    //According to the format of the specification the corresponding constructor
    //is invoked.
    size_t pos = fileName.find('.');
    std::string fileExtension = fileName.substr(pos+1);

    if(fileExtension == "lpp")
    {
        LPPsystem_p lppParser;    //  Our parser

        //Generate a string with the content of the file
        std::string stringFile;
        std::string line;

        ifstream inputFile(fileName.c_str());

        if (inputFile.is_open())
        {
            while (! inputFile.eof() )
            {
                getline (inputFile,line);
                //cout << line << endl;
                stringFile += line;
            }

            inputFile.close();
        }
        else
        {
            cerr << "Unable to open file 4" << std::endl;
            exit(EXIT_FAILURE);
        }

        //We parse the content of the file to generate a abstract syntax tree
        tree_parse_info<> result = ast_parse(stringFile.c_str(),lppParser,space_p);

        if (result.full)
        {
            //cout << "-------------------------\n";
            //cout << "Parsing succeeded " << fileName << std::endl;
            //cout << "-------------------------\n";

            //When the parsing is succesfull the xml file is generated
            iter_t iter = result.trees.begin();
            xmlFile = generateLPPXML(iter);

        }
        else
        {
                //Failed parsing
                cerr << "Parsing of file " << fileName << " according to BNF failed at: " << std::endl;
                std::cout << result.stop << std::endl;
                exit(EXIT_FAILURE);
        }
    }
    //Specification in xml format
    else if(fileExtension == "xml")
    {
        xmlFile = fileName;
    }




  //Use of the DOM parser
  DomParser parser;

  //TODO: Validate our XML file.

  //Parse our XML file.
  parser.parse_file(xmlFile);

  if(!parser)
    {
      cerr << "Parsing Unsuccesfull" << std::endl;
      exit(EXIT_FAILURE);
    }

  //Get the root node of the CAP system Model
  xmlpp::Node* modelDeclaration = parser.get_document()->get_root_node();

  //We get the name of the model
  xmlpp::Element* model = dynamic_cast<xmlpp::Element*>(modelDeclaration);
  if(model->get_attribute("name") == NULL)
  {
      cerr << "Model without name" << std::endl;
      exit(EXIT_FAILURE);
  }
  else
      this->name = (model->get_attribute("name"))->get_value();

  //Get the declaration of the different components of the model
  xmlpp::Node::NodeList components = modelDeclaration->get_children();
  xmlpp::Node::NodeList pSystemsDeclaration;
  xmlpp::Node* latticeDeclaration = NULL;
  xmlpp::Node* cellularAutomataDeclaration = NULL;

  for(xmlpp::Node::NodeList::iterator comp = components.begin(); comp != components.end(); ++comp)
    {
      if((*comp)->get_name() == "listOfPSystems")
	{
	  pSystemsDeclaration = (*comp)->get_children("PSystem");
	}
      else if ((*comp)->get_name() == "lattice")
	{
	  latticeDeclaration = *comp;
	}
      else if ((*comp)->get_name() == "cellularAutomata")
	{
	  cellularAutomataDeclaration = *comp;
	}
    }

	// sanity check
	if(pSystemsDeclaration.empty()) {
		cerr << "error: no P system definition\n";
		exit(EXIT_FAILURE);
	}
	if(!latticeDeclaration) {
		cerr << "error: no lattice definition\n";
		exit(EXIT_FAILURE);
	}
	if(!cellularAutomataDeclaration) {
		cerr << "error: no cellular automata definition\n";
		exit(EXIT_FAILURE);
	}

  //Construct the different components from their declarations

  //Construct the list of P systems
  this->pSystems = createPSystems(pSystemsDeclaration);


//   for(std::map<std::string,PSystem>::iterator p = (this->pSystems).begin();
//   p != (this->pSystems).end();
//   ++p)
//   {
//       (p->second).show();
//   }
//
//   std::string pause;
//   cin >> pause;


  //Construct the lattice
  xmlpp::Element* latticeElement = dynamic_cast<xmlpp::Element*>(latticeDeclaration);
  xmlpp::Attribute* latticeFile = latticeElement->get_attribute("file");

  if(latticeFile != NULL)
  {
      std::string latticeFileName = latticeFile->get_value();

      lattice lat(latticeFileName);
      //parser.get_document()->get_root_node());
      this->spatialLattice = lat;

  }
  else
  {
      //lattice lat(latticeDeclaration);
      //this->spatialLattice = lat;
  }

  //Construct the cellular automata
  CellularAutomata cells(cellularAutomataDeclaration, this->pSystems);

  //Mutate the cellular automata according to the noise if specified
  double noise = cells.getNoise();

  if (noise > 0)
  {
    int numberMutations = cells.getNumberMutations();

    for(int i = 0; i < numberMutations; ++i)
/*        std::cout << "Applying noise: " << i << std::endl;*/
        cells.mutate(noise,0,(this->spatialLattice).getXMax(),0,(this->spatialLattice).getYMax());
  }

  this->cellAutomata = cells;


}

//Create the list of P system distributed over the system
std::map<std::string, ECSB::PSystem> CAPSystem::createPSystems(xmlpp::Node::NodeList pSystemsDeclaration)
{

  std::map<std::string, ECSB::PSystem> result;

  for(xmlpp::Node::NodeList::iterator pSyst = pSystemsDeclaration.begin();
      pSyst != pSystemsDeclaration.end();
      ++pSyst)
    {

      PSystem pSystem;

        //Name of the PSystem and attributes for the specification mode
        xmlpp::Element* pSystemElement = dynamic_cast<xmlpp::Element*>(*pSyst);
        xmlpp::Attribute* fileName = pSystemElement->get_attribute("file");
        xmlpp::Attribute* pSystemName = pSystemElement->get_attribute("name");
        xmlpp::Attribute* pSystemFormat = pSystemElement->get_attribute("format");

        //We distinguish between the option of specifying the P system in a separte file
        //or in this file. In the first case an attribute with the name of the file is provided.
        std::string pSystemFile;
        if(fileName != NULL)
        {
            pSystemFile = fileName->get_value();
        }
        else
        {
            pSystem = PSystem(*pSyst);
        }

        //According to the format of the specification the corresponding constructor
        //is invoked.
        size_t pos = pSystemFile.find('.');
        std::string fileExtension = pSystemFile.substr(pos+1);

        //Specification in text file according to our BNF grammar. We need to
        //convert it into an XML file and then pass it to the corresponding
        //constructor
        if(fileExtension == "sps")
        {
            PSModel pars;    //  Our parser

            //Generate a string with the content of the file
            std::string stringFile;
            std::string line;

            ifstream inputFile(pSystemFile.c_str());

            if (inputFile.is_open())
            {
                while (! inputFile.eof() )
                {
                getline (inputFile,line);
                //cout << line << endl;
                stringFile += line;
                }

                inputFile.close();
            }
            else
            {
                cerr << "Unable to open file" << pSystemFile << std::endl;
                exit(EXIT_FAILURE);
            }

            //We parse the content of the file to generate a abstract syntax tree
            tree_parse_info<> result = ast_parse(stringFile.c_str(),pars,space_p);

            if (result.full)
            {
                //cout << "-------------------------\n";
                //cout << "P system parsing succeeded \n";
                //cout << "-------------------------\n";
                
                //When the parsing is succesfull the xml file is generated
                iter_t iter = result.trees.begin();
                std::string xmlFile = generateXML(iter);

                //Use of the DOM parser
                DomParser parser;
                parser.parse_file(xmlFile);

                if(!parser)
                {
                    cerr << "Parsing Unsuccesfull" << std::endl;
                    exit(EXIT_FAILURE);
                }

                pSystem = PSystem(parser.get_document()->get_root_node());
            }
            else
            {
                //Failed parsing
                cerr << "Parsing of file " << pSystemFile << " according to BNF failed at: " << std::endl;
                std::cout << result.stop << std::endl;
                exit(EXIT_FAILURE);
            }
        }
        //Specification in xml format
        else if(fileExtension == "xml")
        {
            //Use of the DOM parser
            DomParser parser;
            //TODO: Validate our XML file.
            //Parse our XML file.

            parser.parse_file(pSystemFile);

            if(!parser)
            {
                cerr << "Parsing Unsuccesfull" << std::endl;
                exit(EXIT_FAILURE);
            }

            pSystem = PSystem(parser.get_document()->get_root_node());
        }
        //Specification in sbml format
        else if(fileExtension == "sbml")
        {
            //Read the sbml file
            SBMLDocument* sbmlDocument = readSBML(pSystemFile.c_str());
            pSystem = PSystem(sbmlDocument);
        }
        else
        {
            cerr << "Error: Unknown format in file " << pSystemFile << std::endl << "Supported formats: sps, xml and sbml" << std::endl;
            exit(EXIT_FAILURE);
        }

/*

        if(fileName != NULL)
        {
            std::string pSystemFile = fileName->get_value();
            std::string specificationFormat;

            if(pSystemFormat == NULL)
            {
                specificationFormat = "xml";
            }
            else
            {
                specificationFormat = pSystemFormat->get_value();
            }

            if(specificationFormat == "xml")
            {
                //Use of the DOM parser
                DomParser parser;
                //TODO: Validate our XML file.
                //Parse our XML file.

                parser.parse_file(pSystemFile);

                if(!parser)
                {
                    cerr << "Parsing Unsuccesfull" << std::endl;
                    exit(EXIT_FAILURE);
                }

                pSystem = PSystem(parser.get_document()->get_root_node());
            }
            else if (specificationFormat == "sbml")
            {
                pSystem = PSystem(pSystemFile);
            }
            else
            {
                cerr << "Incorrect format specification: " << pSystemFormat->get_value() << std::endl;
                exit(EXIT_FAILURE);
            }

            pSystem.setName(pSystemName->get_value());
        }
        else
        {
            pSystem = PSystem(*pSyst);
        }

 *
 */

      //We set the name of the P system according to the name in the LPP system.
      pSystem.setName(pSystemName->get_value());

      //Insertion of the newly created P system
      result.insert(pair<std::string,PSystem>(pSystem.getName(),pSystem));
    }

  return result;
}

//Default Destructor
CAPSystem::~CAPSystem()
{

}

//Getters
std::string CAPSystem::getName()
{
  return this->name;
}

std::vector<std::string> CAPSystem::getCombinedAlphabets()
{

  std::list<std::string> temporalList;

  for(std::map<std::string,PSystem>::iterator pSyst = this->pSystems.begin();
      pSyst != this->pSystems.end();
      ++pSyst)
    {
      std::vector<std::string> pSystemAlphabet = (pSyst->second).getAlphabet();

      for(std::vector<std::string>::iterator o = pSystemAlphabet.begin();
	  o != pSystemAlphabet.end();
	  ++o)
	{
	  temporalList.push_back(*o);
	}
    }

  temporalList.sort();
  temporalList.unique();

  std::vector<std::string> combinedAlphabets(temporalList.begin(),temporalList.end());

  return combinedAlphabets;

}


std::map<std::string,CAPRuleSet> CAPSystem::getCombinedRuleSets()
{

  std::map<std::string,ECSB::CAPRuleSet> combinedRuleSets;

  for(std::map<std::string,PSystem>::iterator pSyst = (this->pSystems).begin();
      pSyst != (this->pSystems).end();
      ++pSyst)
    {
      std::string firstLabel = pSyst->first;
      std::map<std::string,ECSB::CAPRuleSet> ruleSets = (pSyst->second).getRuleSets();
      for(std::map<std::string,ECSB::CAPRuleSet>::iterator rSet = ruleSets.begin();
	  rSet != ruleSets.end();
	  ++rSet)
	{
	  //The global label of the rule set is the combination of the P system name and
	  //it local label
	  std::string secondLabel = rSet->first;
	  std::string labelRuleSet = firstLabel + secondLabel;

	  combinedRuleSets.insert(pair<std::string,ECSB::CAPRuleSet>(labelRuleSet,rSet->second));
	}
    }
  return combinedRuleSets;
}


std::map<std::string,ECSB::InitialMultiset> CAPSystem::getCombinedInitialMultisets()
{

  std::map<std::string,InitialMultiset> combinedInitialMultisets;

  for(std::map<std::string,PSystem>::iterator pSyst = (this->pSystems).begin();
      pSyst != (this->pSystems).end();
      ++pSyst)
    {
      std::string firstLabel = pSyst->first;
      std::map<std::string,ECSB::InitialMultiset> inis = (pSyst->second).getInitialMultisets();
      for(std::map<std::string,ECSB::InitialMultiset>::iterator ini = inis.begin();
	  ini != inis.end();
	  ++ini)
	{
	  //The global label of the rule set is the combination of the P system name and
	  //it local label
	  std::string secondLabel = ini->first;
	  std::string labelInitialMultiset = firstLabel + secondLabel;

	  combinedInitialMultisets.insert(pair<std::string,ECSB::InitialMultiset>
					  (labelInitialMultiset,ini->second));
	}
    }
  return combinedInitialMultisets;
}


unsigned long CAPSystem::getNumberOfCompartments()
{
  unsigned long numberCompartments = 0;

  for(std::map<std::string,PSystem>::iterator pSyst = (this->pSystems).begin();
      pSyst != (this->pSystems).end();
      ++pSyst)
    {
      numberCompartments += (pSyst->second).getNumberOfCompartments();
    }

  return numberCompartments;

}

CellularAutomata CAPSystem::getCellularAutomata()
{
  return this->cellAutomata;
}

std::map<std::string,PSystem> CAPSystem::getPSystems()
{
  return this->pSystems;
}

PSystem CAPSystem::getPSystem(std::string pSystemName)
{
  return (this->pSystems)[pSystemName];
}

//Setters
void CAPSystem::setPSystems(std::map<std::string,PSystem> systems)
{
    this->pSystems = systems;
}

//Displayer
void CAPSystem::show()
{

  for(std::map<std::string,PSystem>::iterator pSys = this->pSystems.begin();
      pSys != this->pSystems.end();
      ++pSys)
    {
      (pSys->second).show();
    }
  this->spatialLattice.show();

  //this->cellAutomata.show();
}


////////////////////////////////////////
// CellularAutomata class definitions //
////////////////////////////////////////

//Default Constructor
CellularAutomata::CellularAutomata()
{

}

//Copy constructor
CellularAutomata::CellularAutomata(const CellularAutomata& cellularAutomata)
{
    this->automata = cellularAutomata.automata;
    this->distribution = cellularAutomata.distribution;
    this->name = cellularAutomata.name;
    this->noise = cellularAutomata.noise;
    this->numberMutations = cellularAutomata.numberMutations;
    this->numberOfCompartments = cellularAutomata.numberOfCompartments;
}


//Default Destructor
CellularAutomata::~CellularAutomata()
{

}

//Constructor from XML specification
CellularAutomata::CellularAutomata(xmlpp::Node* cellAutomataDeclaration, std::map<std::string,PSystem> pSystems)
{
  //Get the name of the cellular automata
  xmlpp::Element* cellAutomataElement = dynamic_cast<xmlpp::Element*>(cellAutomataDeclaration);
  xmlpp::Attribute* cellAutomataName = cellAutomataElement->get_attribute("name");
  if(cellAutomataName != NULL)
    this->name = cellAutomataName->get_value();

  //Get the type of distribution of the automata. Fixed is the default distribution
  xmlpp::Attribute* distributionDeclaration = cellAutomataElement->get_attribute("distribution");
  if(distributionDeclaration != NULL)
      this->distribution = distributionDeclaration->get_value();
  else
      this->distribution = "fixed";

  //Get the noise for mutation (default no noise) and number of mutations (default 10).
  xmlpp::Attribute* noiseDeclaration = cellAutomataElement->get_attribute("noise");
  if(noiseDeclaration != NULL)
      this->noise = fromStringTo<double>(noiseDeclaration->get_value());
  else
      this->noise = 0;

  xmlpp::Attribute* mutationsDeclaration = cellAutomataElement->get_attribute("mutation_runs");
  if(mutationsDeclaration != NULL)
      this->numberMutations = fromStringTo<int>(mutationsDeclaration->get_value());
  else
      this->numberMutations = 10;



  //Get the automata declarations
  xmlpp::Node::NodeList automataDeclaration = cellAutomataDeclaration->get_children("automaton");

  //Construct the automata depending on the type of distribution and count the total
  //number of compartments in the system
  this->numberOfCompartments = 0;

  if(this->distribution == "fixed")
    {
      //Coordinates are fixed
      fixedPositionAutomata(automataDeclaration,pSystems);
    }
  else if(this->distribution == "random")
    {
      //Coordinates are random
      randomPositionAutomata(automataDeclaration,pSystems);
    }
  else
    {
      cerr << "Unknown distribution" << std::endl;
      exit(EXIT_FAILURE);
    }

}

//Automata constructor with fixed positions
void CellularAutomata::fixedPositionAutomata(xmlpp::Node::NodeList automataDeclaration, std::map<std::string,PSystem> pSystems)
{

  for(xmlpp::Node::NodeList::iterator a = automataDeclaration.begin();
      a != automataDeclaration.end();
      ++a)
    {
      //Get the information about the automaton
      xmlpp::Element* automatonElement = dynamic_cast<xmlpp::Element*>(*a);
      std::string pSystemName = (automatonElement->get_attribute("pSystem"))->get_value();

      std::map<std::string,PSystem>::iterator pSystem = pSystems.find(pSystemName);

      if(pSystem == pSystems.end())
      {
          cerr<< "Error: P system " << pSystemName << " not defined" << std::endl;
          exit(EXIT_FAILURE);
      }

      //Constructing the coordinates for the automata with the same P system
      std::vector< std::vector<int> > coordinatesPrecursor;

      //Getting the parameters associated with the positions
      xmlpp::Node::NodeList parametersDeclaration = (*a)->get_children("parameter");
      std::vector<ECSB::parameter> parameters;

      for(xmlpp::Node::NodeList::iterator param = parametersDeclaration.begin();
      param != parametersDeclaration.end();
      ++param)
      {
          parameters.push_back(parameter(*param));
      }

      //Creating the coordinates
      xmlpp::Node::NodeList xCoordinateDeclaration = (*a)->get_children("Xcoordinate");
      xmlpp::Node::NodeList yCoordinateDeclaration = (*a)->get_children("Ycoordinate");

      //Getting the declaration of the coordinates
      xmlpp::Node* xCoordinate = xCoordinateDeclaration.front();
      xmlpp::Element* xCoordinateElement = dynamic_cast<Element*>(xCoordinate);
      xmlpp::Attribute* xCoordinateValue = xCoordinateElement->get_attribute("value");

      xmlpp::Node* yCoordinate = yCoordinateDeclaration.front();
      xmlpp::Element* yCoordinateElement = dynamic_cast<Element*>(yCoordinate);
      xmlpp::Attribute* yCoordinateValue = yCoordinateElement->get_attribute("value");

      if(!xCoordinateDeclaration.empty())
	{
	  std::vector<int> xCoordinates;
	  if(xCoordinateValue != NULL)
	    {
              //The x coordinate can be expressed as a numeric value or using a mathematical expression
              if(isNumeric(xCoordinateValue->get_value()))
              {
                  //xCoordinates.push_back(fromStringTo<int>(xCoordinateValue->get_value()));
                  xCoordinates = evaluateExpression(xCoordinateValue->get_value(),parameters);
              }
              else
              {
                  xCoordinates = evaluateExpression(xCoordinateValue->get_value(),parameters);
              }
	    }
	  else
	    {
	      int xLow = fromStringTo<int>((xCoordinateElement->get_attribute("lowerBound"))->get_value());
	      int xUpper = fromStringTo<int>((xCoordinateElement->get_attribute("upperBound"))->get_value());
	      for(int i =xLow; i <= xUpper; ++i)
		xCoordinates.push_back(i);
	    }
	  coordinatesPrecursor.push_back(xCoordinates);
	}


      if(!yCoordinateDeclaration.empty())
	{
	  std::vector<int> yCoordinates;
	  if(yCoordinateValue != NULL)
	    {
              //The x coordinate can be expressed as a numeric value or using a mathematical expression
              if(isNumeric(yCoordinateValue->get_value()))
              {
                  //yCoordinates.push_back(fromStringTo<int>(yCoordinateValue->get_value()));
                  yCoordinates = evaluateExpression(yCoordinateValue->get_value(),parameters);
              }
              else
              {
                  yCoordinates = evaluateExpression(yCoordinateValue->get_value(),parameters);
              }
            }
	  else
	    {
	      int yLow = fromStringTo<int>((yCoordinateElement->get_attribute("lowerBound"))->get_value());
	      int yUpper = fromStringTo<int>((yCoordinateElement->get_attribute("upperBound"))->get_value());
	      for(int i =yLow; i <= yUpper; ++i)
		yCoordinates.push_back(i);
	    }

	  coordinatesPrecursor.push_back(yCoordinates);
	}


      std::vector< std::vector<int> > coordinatesFinal;
      //Depending on how the precursors were obtained we generate the final coordinates
      if((xCoordinateValue != NULL) && (yCoordinateValue != NULL))
      {
          coordinatesFinal = combineParametricCoordinates(coordinatesPrecursor);
      }
      else
      {
          coordinatesFinal = combineCoordinates(coordinatesPrecursor);
      }

      //Getting the initial multiset if defined
      xmlpp::Node::NodeList multisetList = (*a)->get_children("initialMultiset");

      if(!multisetList.empty())
      {
          xmlpp::Node* multisetDeclaration = multisetList.front();
          InitialMultiset multiset(multisetDeclaration);
          //Assign coordinates and multiset to automata
          for(std::vector< std::vector<int> >::iterator coord = coordinatesFinal.begin();
            coord != coordinatesFinal.end();
            ++coord)
            {
                automaton cellAutomaton(*coord,pSystemName,multiset);

                this->automata.push_back(cellAutomaton);
            }
      }
      else
      {
          //Assign coordinates to automata
          for(std::vector< std::vector<int> >::iterator coord = coordinatesFinal.begin();
            coord != coordinatesFinal.end();
            ++coord)
            {
                automaton cellAutomaton(*coord,pSystemName);
                this->automata.push_back(cellAutomaton);
                this->numberOfCompartments += (pSystem->second).getNumberOfCompartments();
            }
      }
    }
}


void CellularAutomata::randomPositionAutomata(xmlpp::Node::NodeList automataDeclaration, std::map<std::string,PSystem> pSystems)
{

  //In random position automata there will be two P systems. The first one represents a cell
  //that is distributed randomly over the lattice. The remaining of the positions are filled with
  //a P system that represents the environment.
  xmlpp::Node::NodeList::iterator pSystemDeclaration = automataDeclaration.begin();

  //Distribute P systems over the lattice
  //Get the information about the automaton
  xmlpp::Element* psystemAutomatonElement = dynamic_cast<xmlpp::Element*>(*pSystemDeclaration);

  std::string pSystemName;
  if((psystemAutomatonElement->get_attribute("pSystem")) != NULL)
    pSystemName = (psystemAutomatonElement->get_attribute("pSystem"))->get_value();
  else
    {
      std::cout << "No P system specified for automaton" << std::endl;
      exit(1);
    }
  std::map<std::string,PSystem>::iterator pSystem = pSystems.find(pSystemName);

  if(pSystem == pSystems.end())
  {
      cerr<< "Error: P system " << pSystemName << " not defined" << std::endl;
      exit(EXIT_FAILURE);
  }

  this->numberOfCompartments += (pSystem->second).getNumberOfCompartments();

  std::string randomCopies;
  int numberCopies;

  if((psystemAutomatonElement->get_attribute("copies")) != NULL)
    {
      randomCopies = (psystemAutomatonElement->get_attribute("copies"))->get_value();
      numberCopies = fromStringTo<int>(randomCopies);
    }
  else
    {
      std::cout << "Number of copies not specified in random distribution" << std::endl;
      exit(1);
    }

  //Obtain all possible coordinates
  std::vector< std::vector<int> > coordinatesPrecursor;
  xmlpp::Node::NodeList xCoordinateDeclaration = (*pSystemDeclaration)->get_children("Xcoordinate");
  xmlpp::Node::NodeList yCoordinateDeclaration = (*pSystemDeclaration)->get_children("Ycoordinate");

  if(!xCoordinateDeclaration.empty())
    {
      xmlpp::Node* xCoordinate = xCoordinateDeclaration.front();
      xmlpp::Element* xCoordinateElement = dynamic_cast<Element*>(xCoordinate);
      xmlpp::Attribute* xCoordinateValue = xCoordinateElement->get_attribute("value");

      std::vector<int> xCoordinates;
      if(xCoordinateValue != NULL)
	{
	  xCoordinates.push_back(fromStringTo<int>(xCoordinateValue->get_value()));
	}
      else
	{
	  int xLow = fromStringTo<int>((xCoordinateElement->get_attribute("lowerBound"))->get_value());
	  int xUpper = fromStringTo<int>((xCoordinateElement->get_attribute("upperBound"))->get_value());
	  for(int i =xLow; i <= xUpper; ++i)
	    xCoordinates.push_back(i);
	}
      coordinatesPrecursor.push_back(xCoordinates);
    }

  if(!yCoordinateDeclaration.empty())
    {
      xmlpp::Node* yCoordinate = yCoordinateDeclaration.front();
      xmlpp::Element* yCoordinateElement = dynamic_cast<Element*>(yCoordinate);
      xmlpp::Attribute* yCoordinateValue = yCoordinateElement->get_attribute("value");

      std::vector<int> yCoordinates;
      if(yCoordinateValue != NULL)
	{
	  yCoordinates.push_back(fromStringTo<int>(yCoordinateValue->get_value()));
	}
      else
	{
	  int yLow = fromStringTo<int>((yCoordinateElement->get_attribute("lowerBound"))->get_value());
	  int yUpper = fromStringTo<int>((yCoordinateElement->get_attribute("upperBound"))->get_value());
	  for(int i =yLow; i <= yUpper; ++i)
	    yCoordinates.push_back(i);
	}
      coordinatesPrecursor.push_back(yCoordinates);
    }

  std::vector< std::vector<int> > coordinatesFinal = combineCoordinates(coordinatesPrecursor);

  std::vector< std::vector <std::vector<int> > > randomCoordinates = combineRandomCoordinates(coordinatesPrecursor,numberCopies);


  std::vector< std::vector<int> > pSystemCoordinates = randomCoordinates[0];
  std::vector< std::vector<int> > environmentCoordinates = randomCoordinates[1];

  //Distributing P systems
  for(std::vector< std::vector<int> >::iterator coord = pSystemCoordinates.begin();
      coord != pSystemCoordinates.end();
      ++coord)
    {
      automaton cellAutomaton(*coord,pSystemName);
      this->automata.push_back(cellAutomaton);
    }

  //Distributing the environments
  xmlpp::Node::NodeList::iterator environmentDeclaration = ++automataDeclaration.begin();

  //Get the information about the automaton
  xmlpp::Element* environmentAutomatonElement = dynamic_cast<xmlpp::Element*>(*environmentDeclaration);
  std::string environmentName = (environmentAutomatonElement->get_attribute("pSystem"))->get_value();

  for(std::vector< std::vector<int> >::iterator coord = environmentCoordinates.begin();
      coord != environmentCoordinates.end();
      ++coord)
    {
      automaton cellAutomaton(*coord,environmentName);
      this->automata.push_back(cellAutomaton);
    }

}

std::vector< std::vector<int> > CellularAutomata::combineCoordinates(std::vector< std::vector<int> > coordinatesPrecursor)
{

  std::vector< std::vector<int> > combinedCoordinates;
  std::vector<int> xCoordinates = coordinatesPrecursor[0];
  std::vector<int> yCoordinates = coordinatesPrecursor[1];

  for(std::vector<int>::iterator xCoord = xCoordinates.begin();
      xCoord != xCoordinates.end();
      ++xCoord)
    {
      for(std::vector<int>::iterator yCoord = yCoordinates.begin();
	  yCoord != yCoordinates.end();
	  ++yCoord)
	{
	  std::vector<int> coordinate;
	  coordinate.push_back(*xCoord);
	  coordinate.push_back(*yCoord);
	  combinedCoordinates.push_back(coordinate);
	}
    }

  return combinedCoordinates;

}


std::vector< std::vector<int> > CellularAutomata::combineParametricCoordinates(std::vector< std::vector<int> > coordinatesPrecursor)
{

  std::vector< std::vector<int> > combinedCoordinates;
  std::vector<int> xCoordinates = coordinatesPrecursor[0];
  std::vector<int> yCoordinates = coordinatesPrecursor[1];

  for(int i=0; i < xCoordinates.size(); ++i)
  {
      std::vector<int> coordinate;
      coordinate.push_back(xCoordinates[i]);
      coordinate.push_back(yCoordinates[i]);
      combinedCoordinates.push_back(coordinate);
  }

  return combinedCoordinates;

}


std::vector< std::vector <std::vector<int> > > CellularAutomata::combineRandomCoordinates(std::vector< std::vector<int> > coordinatesPrecursor, int randomCopies)
{

  std::vector< std::vector<int> > combinedRandomCoordinates;
  std::vector <std::vector<int> > combinedCoordinates;

  std::vector<int> xCoordinates = coordinatesPrecursor[0];
  std::vector<int> yCoordinates = coordinatesPrecursor[1];

  for(std::vector<int>::iterator xCoord = xCoordinates.begin();
      xCoord != xCoordinates.end();
      ++xCoord)
    {
      for(std::vector<int>::iterator yCoord = yCoordinates.begin();
	  yCoord != yCoordinates.end();
	  ++yCoord)
	{
	  std::vector<int> coordinate;
	  coordinate.push_back(*xCoord);
	  coordinate.push_back(*yCoord);
	  combinedCoordinates.push_back(coordinate);
	}
    }

  //Pick random coordinates
  srand((unsigned)time(0));

  std::vector< std::vector<int> >::iterator coordinatesIterator = combinedCoordinates.begin();
  for(int k = 0; k < randomCopies; ++k)
    {
      int numberCoordinates = combinedCoordinates.size();
      int randomPosition = rand()%numberCoordinates;
      combinedRandomCoordinates.push_back(combinedCoordinates[randomPosition]);
      combinedCoordinates.erase(coordinatesIterator+randomPosition);
    }

  std::vector< std::vector <std::vector<int> > > coordinatesFinal;
  coordinatesFinal.push_back(combinedRandomCoordinates);
  coordinatesFinal.push_back(combinedCoordinates);
  return coordinatesFinal;

}

std::vector< std::vector<double> > CellularAutomata::combineValues(std::vector< std::vector<double> > separateValues)
{
    std::vector< std::vector<double> > combinedValues;

    //Initialising results with vector with a single value corresponding to the range for the first parameter
    std::vector<double> value;
    value.push_back(0);

    for(std::vector<double>::iterator val = (separateValues[0]).begin();
        val != (separateValues[0].end());
        ++val)
    {
        value[0]=*val;
        combinedValues.push_back(value);
    }

    for(std::vector< std::vector<double> >::iterator parameterValues = separateValues.begin()+1;
        parameterValues != separateValues.end();
        ++parameterValues)
    {

        std::vector< std::vector<double> > partialCombinedValues;

        for(std::vector< std::vector<double> >::iterator previousValues = combinedValues.begin();
            previousValues != combinedValues.end();
            ++previousValues)
        {
            for(std::vector<double>::iterator value = parameterValues->begin();
                value != parameterValues->end();
                ++value)
            {
                std::vector<double> newValues = *previousValues;
                newValues.push_back(*value);
                partialCombinedValues.push_back(newValues);
            }
        }
        combinedValues = partialCombinedValues;
    }

    return combinedValues;
}


std::vector<int> CellularAutomata::evaluateExpression(std::string expression, std::vector<ECSB::parameter> parameters)
{
    std::vector<int> result;

	if(parameters.empty()) {
		result.push_back(fromStringTo<int>(expression));
		return result;
	}

	//Create array containing the values for the parameters
    int numberParameters = parameters.size();
    double variables[numberParameters];

    mu::Parser parser;
    //Set the expression to evaluate
    parser.SetExpr(expression);

    //Index of the variable used for the parameter
    int index = 0;

    //Create the variables in the expression
    for(std::vector<ECSB::parameter>::iterator param = parameters.begin();
        param != parameters.end();
        ++param)
    {
        parser.DefineVar(param->getName(),&variables[index]);
        ++index;
    }

    //Generate all possible values for the parameters
    std::vector< std::vector<double> > separateValues;

    for(std::vector<ECSB::parameter>::iterator param = parameters.begin();
        param != parameters.end();
        ++param)
    {
        std::vector<double> parameterValues;

        double paramValue = param->getLowerBound();
        while(paramValue <= param->getUpperBound())
        {
            parameterValues.push_back(paramValue);
            paramValue = paramValue + param->getStep();
        }

        separateValues.push_back(parameterValues);
    }

    //Combine all the possible values of the different parameters
    std::vector< std::vector<double> > values = combineValues(separateValues);

    //Evaluating the expression for all the possible values of the parameters
    for(std::vector< std::vector<double> >::iterator val = values.begin();
        val != values.end();
        ++val)
    {
        std::vector<double> currentValues = *val;
        //Instantiate the current values in the array of variables
        for(int i = 0; i < numberParameters; ++i)
        {
            variables[i] = currentValues[i];
        }

        //Evaluate the expression and add it to the result
        result.push_back(static_cast<int>(parser.Eval()));
    }

    return result;

}


std::map<std::vector<int>,ECSB::automaton> CellularAutomata::convertToMap( )
{

    std::map<std::vector<int>,ECSB::automaton> mapAutomata;

    for(std::vector<automaton>::iterator cell = (this->automata).begin();
        cell != (this->automata).end();
        ++cell)
    {
        std::vector<int> position = cell->getPosition();
        mapAutomata.insert(pair<std::vector<int>,ECSB::automaton>(position,*cell));
    }

    return mapAutomata;
}

void CellularAutomata::mutate(double probability, int xLowerValue, int xUpperValue, int yLowerValue, int yUpperValue)
{
    //Initialise seed
    srand (time(NULL));

    std::vector<ECSB::automaton> mutatedAutomata;
    std::map<std::vector<int>,ECSB::automaton> mapAutomata = convertToMap();

    //Add the borders without mutations
    for(int coord = xLowerValue; coord <= xUpperValue; ++coord)
    {
        //Lower border
        std::vector<int> position1;
        position1.push_back(coord);
        position1.push_back(yLowerValue);

        mutatedAutomata.push_back(mapAutomata[position1]);
        mapAutomata.erase(position1);

        //Upper border
        std::vector<int> position2;
        position2.push_back(coord);
        position2.push_back(yUpperValue);

        mutatedAutomata.push_back(mapAutomata[position2]);
        mapAutomata.erase(position2);
    }

    for(int coord = yLowerValue + 1; coord <= yUpperValue - 1; ++coord)
    {
        //Left border
        std::vector<int> position1;
        position1.push_back(xLowerValue);
        position1.push_back(coord);

        mutatedAutomata.push_back(mapAutomata[position1]);
        mapAutomata.erase(position1);

        //Right border
        std::vector<int> position2;
        position2.push_back(xUpperValue);
        position2.push_back(coord);

        mutatedAutomata.push_back(mapAutomata[position2]);
        mapAutomata.erase(position2);
    }


    while(!mapAutomata.empty())
    {
        std::map<std::vector<int>,ECSB::automaton>::iterator mapAut = mapAutomata.begin();
        std::vector<int> pos1 = mapAut->first;
        ECSB::automaton aut1 = mapAut->second;
        std::string pSystem1 = aut1.getPSystemName();

        double randomNumber = (double)rand()/RAND_MAX;

        if (randomNumber <= probability)
        {
            std::vector<int> pos2 = neighbouringPosition(pos1);
            std::map<std::vector<int>,ECSB::automaton>::iterator neigh = mapAutomata.find(pos2);

            if(neigh == mapAutomata.end())
            {
                mapAutomata.erase(pos1);
                mutatedAutomata.push_back(aut1);
            }
            else
            {
                //swap P systems
                ECSB::automaton aut2 = neigh->second;
                std::string pSystem2 = aut2.getPSystemName();
                aut1.setPSystemName(pSystem2);
                aut2.setPSystemName(pSystem1);
                mutatedAutomata.push_back(aut1);
                mutatedAutomata.push_back(aut2);

                //Remove automata from the map
                mapAutomata.erase(pos1);
                mapAutomata.erase(pos2);
            }

        }
        else
        {
            mapAutomata.erase(pos1);
            mutatedAutomata.push_back(aut1);
        }
    }


    this->automata = mutatedAutomata;

}


std::vector<int> CellularAutomata::neighbouringPosition(std::vector<int> pos)
{
    std::vector<int> result;

    double randomNumber = (double)rand()/RAND_MAX;

    if (randomNumber <= 0.25)
    {
        result.push_back(pos[0] + 1);
        result.push_back(pos[1] + 1);
    }
    else if ((randomNumber > 0.25) && (randomNumber <= 0.5))
    {
        result.push_back(pos[0] - 1);
        result.push_back(pos[1] + 1);
    }
    else if ((randomNumber > 0.5) && (randomNumber <= 0.75))
    {
        result.push_back(pos[0] + 1);
        result.push_back(pos[1] - 1);
    }
    else if ((randomNumber > 0.75) && (randomNumber <= 1))
    {
        result.push_back(pos[0] - 1);
        result.push_back(pos[1] - 1);
    }


    return result;
}

void CellularAutomata::show()
{

  std::cout << "===============================" << std::endl;
  std::cout << "Cellular Automata" << std::endl;
  for(std::vector<automaton>::iterator cellAutomaton = (this->automata).begin();
      cellAutomaton != (this->automata).end();
      ++cellAutomaton)
    {
      cellAutomaton->show();
    }
}

//Getters
std::vector<automaton> CellularAutomata::getAutomata()
{
  return this->automata;
}

double CellularAutomata::getNoise()
{
    return this->noise;
}

int CellularAutomata::getNumberMutations()
{
    return this->numberMutations;
}

unsigned long CellularAutomata::getNumberOfCompartments()
{
    return this->numberOfCompartments;
}

/////////////////////////////////////
// Parameter class implementations //
////////////////////////////////////

parameter::parameter()
{

}

parameter::~parameter()
{

}

//Default copy constructor
parameter::parameter(const parameter& param)
{
    this->lowerBound = param.lowerBound;
    this->name = param.name;
    this->step = param.step;
    this->upperBound = param.upperBound;
}

parameter::parameter(xmlpp::Node* parameterDeclaration)
{

  //Instanciate the different members of a parameter
  xmlpp::Element* param = dynamic_cast<xmlpp::Element*>(parameterDeclaration);

  //Name
  xmlpp::Attribute* nameAttribute = param->get_attribute("name");
  if(nameAttribute != NULL)
  {
      this->name = nameAttribute->get_value();
  }
  else
  {
      cerr<< "Error: parameter without name" <<std::endl;
      exit(EXIT_FAILURE);
  }

  //Lower Bound
  xmlpp::Attribute* lowerBoundAttribute = param->get_attribute("lowerBound");
  if(lowerBoundAttribute != NULL)
  {
      this->lowerBound = fromStringTo<int>(lowerBoundAttribute->get_value());
  }
  else
  {
      cerr<< "Error: parameter without lower bound" <<std::endl;
      exit(EXIT_FAILURE);
  }

  //Upper Bound
  xmlpp::Attribute* upperBoundAttribute = param->get_attribute("upperBound");
  if(upperBoundAttribute != NULL)
  {
      this->upperBound = fromStringTo<int>(upperBoundAttribute->get_value());
  }
  else
  {
      cerr<< "Error: parameter without upper bound" <<std::endl;
      exit(EXIT_FAILURE);
  }

  //Step
  xmlpp::Attribute* stepAttribute = param->get_attribute("step");
  if(stepAttribute != NULL)
  {
      this->step = fromStringTo<int>(stepAttribute->get_value());
  }
  else
  {
      cerr<< "Error: parameter without step" <<std::endl;
      exit(EXIT_FAILURE);
  }

}


//Getters
std::string parameter::getName()
{
    return this->name;
}

int parameter::getLowerBound()
{
    return this->lowerBound;
}

int parameter::getUpperBound()
{
    return this->upperBound;
}

int parameter::getStep()
{
    return this->step;
}

//////////////////////////////////////
// CAPCompartment class definitions //
//////////////////////////////////////

CAPCompartment::CAPCompartment()
{

}

CAPCompartment::~CAPCompartment()
{

}

//Copy constructor
CAPCompartment::CAPCompartment(const CAPCompartment& comp)
{
    this->name = comp.name;
    this->numberId = comp.numberId;
    this->label = comp.label;
    this->father = comp.father;
    this->children = comp.children;
    this->numberId = comp.numberId;
}


CAPCompartment::CAPCompartment(xmlpp::Node* compartmentDeclaration)
{
  //Construct the different components of a compartment
  xmlpp::Element* compartment = dynamic_cast<xmlpp::Element*>(compartmentDeclaration);
  this->name = (compartment->get_attribute("name"))->get_value();
  this->label = (compartment->get_attribute("label"))->get_value();
  xmlpp::Attribute* fatherDeclaration = compartment->get_attribute("father");

  //Father is an optional attribute
  if(fatherDeclaration != NULL)
    this->father = fatherDeclaration->get_value();
}

string CAPCompartment::getName()
{
    return this->name;
}

string CAPCompartment::getLabel()
{
    return this->label;
}

string CAPCompartment::getFather()
{
    return this->father;
}

std::vector<std::string> CAPCompartment::getChildren()
{
    return this->children;
}

unsigned long CAPCompartment::getNumberId()
{
    return this->numberId;
}

//Setters
void CAPCompartment::setName(std::string compartmentName)
{
    this->name = compartmentName;
}

void CAPCompartment::setLabel(std::string compartmentLabel)
{
    this->label = compartmentLabel;
}

void CAPCompartment::setFather(std::string compartmentFather)
{
    this->father = compartmentFather;
}

void CAPCompartment::setChildren(std::vector<std::string> childrenNames)
{
    this->children = childrenNames;
}

void CAPCompartment::setNumberId(unsigned long id)
{
    this->numberId = id;
}



void CAPCompartment::show()
{
  //std::cout << "-----------------------------" << std::endl;
  std::cout << "\tCompartment " << this->name << " : " << std::endl;
  std::cout << "\t\tlabel: " << this->label << std::endl;
  std::cout << "\t\tfather: " << this->father << std::endl;
  std::cout << "\t\tchildren: " << std::endl;

  for(std::vector<std::string>::iterator ch = this->children.begin();
    ch != this->children.end();
    ++ch)
  {
      std::cout << "\t\t\t" << *ch << std::endl;
  }

  //std::cout << "-----------------------------" << std::endl;
}

///////////////////////////////////////
// InitialMultiset class definitions //
///////////////////////////////////////

InitialMultiset::InitialMultiset()
{

}

InitialMultiset::~InitialMultiset()
{

}

InitialMultiset::InitialMultiset(const InitialMultiset& initialMultiset)
{
    this->label = initialMultiset.label;
    this->multiset = initialMultiset.multiset;
}

InitialMultiset::InitialMultiset(xmlpp::Node* initialMultisetDeclaration)
{
  xmlpp::Element* ini = dynamic_cast<Element*>(initialMultisetDeclaration);

  this->label = (ini->get_attribute("label"))->get_value();

  xmlpp::Node::NodeList objs = initialMultisetDeclaration->get_children("object");
  for(xmlpp::Node::NodeList::iterator obj = objs.begin(); obj != objs.end(); ++obj)
    {
      xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
      std::string objectName = (object->get_attribute("name"))->get_value();
      std::string objectMultiplicity = (object->get_attribute("multiplicity"))->get_value();
      unsigned long multiplicity = fromStringTo<unsigned long> (objectMultiplicity);

      this->multiset.insert(pair<std::string,unsigned long>(objectName,multiplicity));
    }
}

//Getters
std::string InitialMultiset::getLabel(){return this->label;};
std::map<std::string,unsigned long> InitialMultiset::getMultiset(){return this->multiset;};

//Setters
void InitialMultiset::setLabel(std::string labelName)
{
    this->label = labelName;
}

void InitialMultiset::setObjectMultiplicity(std::string objectName, unsigned long objectMultiplicity)
{
    if ((this->multiset).find(objectName) != (this->multiset).end())
    {
        (this->multiset)[objectName] = objectMultiplicity;
    }
    else
    {
        (this->multiset).insert(pair<std::string,unsigned long >(objectName,objectMultiplicity));
    }
}

//Displayer
void InitialMultiset::show()
{

  std::cout << "Multiset: " << this->label << std::endl;
  for(std::map<std::string,unsigned long>::iterator obj = this->multiset.begin();
      obj != multiset.end();
      ++obj)
    {
      std::cout << obj->first << ":" << obj->second << std::endl;
    }
}




////////////////////////////////////////
// ModuleDefinition class definitions //
////////////////////////////////////////

//Default Constructor
ModuleDefinition::ModuleDefinition()
{

}

ModuleDefinition::~ModuleDefinition()
{

}

ModuleDefinition::ModuleDefinition(const ModuleDefinition& moduleDefinition)
{
    this->constants = moduleDefinition.constants;
    this->labels = moduleDefinition.labels;
    this->lowerBounds = moduleDefinition.lowerBounds;
    this->name = moduleDefinition.name;
    this->objects = moduleDefinition.objects;
    this->precisions = moduleDefinition.precisions;
    this->rules = moduleDefinition.rules;
    this->scales = moduleDefinition.scales;
    this->upperBounds = moduleDefinition.upperBounds;
}

//Constructor from XML
ModuleDefinition::ModuleDefinition(xmlpp::Node* moduleDeclaration, std::map<std::string,ModuleDefinition> partialLibrary)
{
  //We down cast the module definition in order to obtain its attributes.
  xmlpp::Element* declaration = dynamic_cast<xmlpp::Element*>(moduleDeclaration);

  //Module name
  this->name = (declaration->get_attribute("name"))->get_value();

  //Module objects
  xmlpp::Node::NodeList setOfObjectsList = moduleDeclaration->get_children("setOfObjects");

  xmlpp::Node* objectsListDeclaration = setOfObjectsList.front();

  xmlpp::Node::NodeList objectsList = objectsListDeclaration->get_children("object");

  for(xmlpp::Node::NodeList::iterator it = objectsList.begin();
      it != objectsList.end();
      ++it)
    {
      xmlpp::Element* objectDeclaration = dynamic_cast<xmlpp::Element*>(*it);
      xmlpp::Attribute* objectName = objectDeclaration->get_attribute("name");
      this->objects.push_back(objectName->get_value());
    }


  //Module constants
  xmlpp::Node::NodeList setOfConstantsList = moduleDeclaration->get_children("setOfConstants");

  xmlpp::Node* constantsListDeclaration = setOfConstantsList.front();

  xmlpp::Node::NodeList constantsList = constantsListDeclaration->get_children("constant");

  for(xmlpp::Node::NodeList::iterator it = constantsList.begin();
      it != constantsList.end();
      ++it)
    {
      xmlpp::Element* constantDeclaration = dynamic_cast<xmlpp::Element*>(*it);
      xmlpp::Attribute* constantName = constantDeclaration->get_attribute("name");
      std::string constantNameValue = constantName->get_value();
      this->constants.push_back(constantNameValue);

      xmlpp::Attribute* constantLowerBound = constantDeclaration->get_attribute("lowerBound");
      if(constantLowerBound != NULL)
	{
	  double constantLowerBoundValue =
	    fromStringTo<double>(constantLowerBound->get_value());
	  this->lowerBounds.insert(pair<string,double>(constantNameValue,constantLowerBoundValue));
	}

      xmlpp::Attribute* constantUpperBound = constantDeclaration->get_attribute("upperBound");
      if(constantUpperBound != NULL)
	{
	  double constantUpperBoundValue = fromStringTo<double>(constantUpperBound->get_value());
	  this->upperBounds.insert(pair<string,double>(constantNameValue,constantUpperBoundValue));
	}

      xmlpp::Attribute* constantPrecision = constantDeclaration->get_attribute("precision");
      if(constantPrecision != NULL)
	{
	  double constantPrecisionValue = fromStringTo<double>(constantPrecision->get_value());
	  this->precisions.insert(pair<string,double>(constantNameValue,constantPrecisionValue));
	}

      xmlpp::Attribute* constantScale = constantDeclaration->get_attribute("scale");
      if(constantScale != NULL)
	{
	  std::string constantScaleValue = constantScale->get_value();
	  this->scales.insert(pair<string,string>(constantNameValue,constantScaleValue));
	}
    }

  //Module labels
  xmlpp::Node::NodeList setOfLabelsList = moduleDeclaration->get_children("setOfLabels");

  xmlpp::Node* labelsListDeclaration = setOfLabelsList.front();
  xmlpp::Node::NodeList labelsList = labelsListDeclaration->get_children("label");

  for(xmlpp::Node::NodeList::iterator it = labelsList.begin();
      it != labelsList.end();
      ++it)
    {
      xmlpp::Element* labelDeclaration = dynamic_cast<xmlpp::Element*>(*it);
      xmlpp::Attribute* labelName = labelDeclaration->get_attribute("name");
      this->labels.push_back(labelName->get_value());
    }

  //Module rules
  xmlpp::Node::NodeList setOfRulesList = moduleDeclaration->get_children("setOfRules");

  xmlpp::Node* rulesListDeclaration = setOfRulesList.front();

  //Get all the nodes with the rules declarations
  xmlpp::Node::NodeList rulesList = rulesListDeclaration->get_children("rule");

  for(xmlpp::Node::NodeList::iterator it = rulesList.begin();
      it != rulesList.end();
      ++it)
    {
      rules.push_back(CAPRule(*it));
    }

  //Get all the nodes with the modules declarations
  xmlpp::Node::NodeList moduleList = rulesListDeclaration->get_children("module");

  for(xmlpp::Node::NodeList::iterator m = moduleList.begin(); m != moduleList.end(); ++m)
    {
      std::vector<CAPRule> moduleRules = expand(*m, partialLibrary);
      rules.insert(rules.end(),moduleRules.begin(),moduleRules.end());
    }

  //Remove the duplicates coming from different modules
  //rules = removeDuplicates(rules);

  //Rename uniquely each rule
  //rules = individualRuleNames(rules,(this->labels)[0]);

}

//Displayer
void ModuleDefinition::showModule()
{

  std::cout << "---------------------------------------" << std::endl;
  std::cout << " Module name: " << this->name << std::endl << std::endl;
  std::cout << " \tObjects: ";

  for(vector<string>::iterator it = this->objects.begin();
      it != this->objects.end();
      ++it)
    {
      std::cout << *it << " ";
    }

  std::cout << endl;

  std::cout << "\tConstants: ";

  for(std::vector<std::string>::iterator it = this->constants.begin();
      it != this->constants.end();
      ++it)
    {
      std::cout << *it << "=" << lowerBounds[*it] << ":" << precisions[*it] << ":";
      std::cout << upperBounds[*it] << " " << "scale:" << scales[*it] << "  ";
    }

  std::cout << std::endl;

  std::cout << "\tLabels: ";

  for(std::vector<std::string>::iterator it = this->labels.begin();
      it != this->labels.end();
      ++it)
    {
      std::cout << *it << " ";
    }

  std::cout << std::endl;

  std::cout << "\tRules: " << std::endl;

  //std::vector<CAPRule> all_rules;

  for(std::vector<CAPRule>::iterator it = this->rules.begin();
      it != this->rules.end();
      ++it)
    {
      std::cout << "\t\t";
      it->showRule();
    }

  std::cout << "---------------------------------------" << std::endl;




}


//Getters
std::string ModuleDefinition::getName() { return this->name; };
std::vector<std::string> ModuleDefinition::getObjects() { return this->objects; };
std::vector<std::string> ModuleDefinition::getConstants() { return this->constants; };
double ModuleDefinition::getLowerBound (std::string c) { return this->lowerBounds[c]; };
double ModuleDefinition::getUpperBound (std::string c) { return this->upperBounds[c]; };
double ModuleDefinition::getPrecision (std::string c) { return this->precisions[c]; };
std::string ModuleDefinition::getScale (std::string c) { return this->scales[c]; };
std::vector<std::string> ModuleDefinition::getLabels() { return this->labels; };
std::vector<CAPRule> ModuleDefinition::getRules() { return this->rules;};

//Setters
void ModuleDefinition::setObjects(std::vector<std::string> newObjects)
{
    this->objects = newObjects;
}

void ModuleDefinition::setRules(std::vector<CAPRule> newRules)
{
    this->rules = newRules;
}


//Instantiating a module definition
std::vector<CAPRule> ModuleDefinition::instantiate(std::map<std::string,std::string> objectsSubstitution,
						std::map<std::string,string> constantsSubstitution,
						std::map<std::string,std::string> labelsSubstitution)
{
    std::vector<CAPRule> instantiatedRules = this->rules;

    for(std::vector<CAPRule>::iterator r = instantiatedRules.begin();
      r != instantiatedRules.end();
      ++r)
    {
        r->substituteObjects(objectsSubstitution);
        r->substituteConstants(constantsSubstitution);
        r->substituteLabels(labelsSubstitution);
    }

    return instantiatedRules;
}

/////////////////////////////////////
// ModuleLibrary class definitions //
/////////////////////////////////////

ModuleLibrary::ModuleLibrary()
{

}

ModuleLibrary::~ModuleLibrary()
{

}

ModuleLibrary::ModuleLibrary(const ModuleLibrary& moduleLibrary)
{
    this->modules = moduleLibrary.modules;
    this->name = moduleLibrary.name;
}



ModuleLibrary::ModuleLibrary(std::string libraryFile)
{
    std::string xmlFile;
    
    //Checking the format of the library according to the extension of the file
    size_t pos = libraryFile.find('.');
    std::string fileExtension = libraryFile.substr(pos+1);

    //Specification in text file according to our BNF grammar. We need to
    //convert it into an XML file and then pass it to the corresponding
    //constructor
    if(fileExtension == "plb")
    {
        library_p libraryParser;    //  Our parser

        //Generate a string with the content of the file
        std::string stringFile;
        std::string line;

        ifstream inputFile(libraryFile.c_str());

        if (inputFile.is_open())
        {
            while (! inputFile.eof() )
            {
                getline (inputFile,line);
                //cout << line << endl;
                stringFile += line;
            }

            inputFile.close();
        }
        else
        {
            cerr << "Unable to open file 2 " << libraryFile << std::endl;
            exit(EXIT_FAILURE);
        }

        //We parse the content of the file to generate a abstract syntax tree
        tree_parse_info<> result = ast_parse(stringFile.c_str(),libraryParser,space_p);

        if (result.full)
        {
            //cout << "-------------------------\n";
            //cout << "Libary Parsing succeeded\n";
            //cout << "-------------------------\n";

            //When the parsing is succesfull the xml file is generated
            iter_t iter = result.trees.begin();
            xmlFile = generateLibraryXML(iter);
        }
        else
        {
            //Failed parsing
            cerr << "Parsing of file " << libraryFile << " according to BNF failed at: " << std::endl;
            std::cout << result.stop << std::endl;
            exit(EXIT_FAILURE);
        }
    }
    //Specification in xml format
    else if(fileExtension == "xml")
    {
        //Use of the DOM parser
        xmlFile = libraryFile;
    }
    else
    {
        cerr << "Library format " << fileExtension << "not supported. Supported formats plb and xml." << std::endl;
        exit(EXIT_FAILURE);
    }


  //Check wether the file exists
  FILE* file = fopen(xmlFile.c_str(), "r");

  if(!file)
    {
      std::cout << "Problems loading library file: " << xmlFile << std::endl;
      exit(1);
    }


  //We use the DOM parser which parses the whole document and stores the structure in memory.
  DomParser parser;

  //To validate our XML file we have to define a DTD (document type definition)
  //parser.set_validate();

  //We parse our XML file.
  parser.parse_file(xmlFile);

  if(!parser)
    {
      cout << "Parsing Unsuccesfull" << endl;
      exit(1);
    }

  //We get the root node of the Module Library
  xmlpp::Node* rootNode = parser.get_document()->get_root_node();

  xmlpp::Node::NodeList children = rootNode->get_children("moduleDefinition");

  for(xmlpp::Node::NodeList::iterator it = children.begin();
      it != children.end();
      ++it)
    {
      ModuleDefinition mod(*it, this->modules);
      this->modules.insert(pair<string,ModuleDefinition>(mod.getName(),mod));
    }

}

void ModuleLibrary::showModuleLibrary()
{

  cout << "================================================" << endl;
  cout << " Module Library: " << name << endl;

  for(map<string,ModuleDefinition>::iterator it = modules.begin(); it != modules.end(); ++it)
    {
      //(it->second).show_Module();
      (it->second).showModule();
    }

  cout << "================================================" << endl;

}

///////////////////////////////
// PSystem class definitions //
///////////////////////////////

PSystem::PSystem()
{

}

//Default Destructor
PSystem::~PSystem()
{

}

//Copy constructor
PSystem::PSystem(const PSystem& pSystem)
{
    this->alphabet = pSystem.alphabet;
    this->initialMultisets = pSystem.initialMultisets;
    this->membraneStructure = pSystem.membraneStructure;
    this->name = pSystem.name;
    this->ruleSets = pSystem.ruleSets;
    this->setOfLabels = pSystem.setOfLabels;
}

PSystem::PSystem(xmlpp::Node* pSystemDeclaration)
{

  //Name of the PSystem
  xmlpp::Element* pSystemElement = dynamic_cast<xmlpp::Element*>(pSystemDeclaration);
  this->name = (pSystemElement->get_attribute("name"))->get_value();

  //Get the declaration of the different components
  xmlpp::Node::NodeList components = pSystemDeclaration->get_children();
  xmlpp::Node::NodeList alphabetDeclaration;
  xmlpp::Node::NodeList labelsDeclaration;
  xmlpp::Node::NodeList compartmentsDeclaration;
  xmlpp::Node::NodeList initialMultisetsDeclaration;
  xmlpp::Node::NodeList ruleSetsDeclaration;

  for(xmlpp::Node::NodeList::iterator comp = components.begin(); comp != components.end(); ++comp)
    {
      if((*comp)->get_name() == "alphabetOfObjects")
	{
	  alphabetDeclaration = (*comp)->get_children("object");
	}
      else if ((*comp)->get_name() == "compartmentLabels")
	{
	  labelsDeclaration = (*comp)->get_children("compartmentLabel");
	}
      else if ((*comp)->get_name() == "membraneStructure")
	{
	  compartmentsDeclaration = (*comp)->get_children("compartment");
	}

      else if ((*comp)->get_name() == "initialMultisets")
	{
	  initialMultisetsDeclaration = (*comp)->get_children("initialMultiset");
	}
      else if ((*comp)->get_name() == "ruleSets")
	{
	  ruleSetsDeclaration = (*comp)->get_children("ruleSet");
	}
    }

  //Construct the different components from their declarations
  createAlphabet(alphabetDeclaration);
  createSetOfLabels(labelsDeclaration);
  createMembraneStructure(compartmentsDeclaration);
  assignChildren();
  createInitialMultisets(initialMultisetsDeclaration);
  createRuleSets(ruleSetsDeclaration);
  checkAlphabet();

}

void PSystem::createAlphabet(xmlpp::Node::NodeList alphabetDeclaration)
{

  for(xmlpp::Node::NodeList::iterator obj = alphabetDeclaration.begin();
      obj != alphabetDeclaration.end();
      ++obj)
    {
      //We get the name of each object and store it
      xmlpp::Element* objElement = dynamic_cast<xmlpp::Element*>(*obj);
      std::string objName = (objElement->get_attribute("name"))->get_value();
      (this->alphabet).push_back(objName);
    }
}

void PSystem::createSetOfLabels(xmlpp::Node::NodeList labelsDeclaration)
{

  for(xmlpp::Node::NodeList::iterator lab = labelsDeclaration.begin();
      lab != labelsDeclaration.end();
      ++lab)
    {
      //We get the name of each label and store it
      xmlpp::Element* labElement = dynamic_cast<xmlpp::Element*>(*lab);
      std::string labName = (labElement->get_attribute("label"))->get_value();
      (this->setOfLabels).push_back(labName);
    }
}

void PSystem::createMembraneStructure(xmlpp::Node::NodeList compartmentsDeclaration)
{
    CAPCompartment compartment;
    unsigned long numberId = 0;

  for(xmlpp::Node::NodeList::iterator comp = compartmentsDeclaration.begin();
      comp != compartmentsDeclaration.end();
      ++comp)
    {
      compartment = CAPCompartment(*comp);
      compartment.setNumberId(numberId);
      numberId++;

      (this->membraneStructure).insert(pair<std::string,CAPCompartment>(compartment.getName(),compartment));
    }

}

std::vector<std::string> PSystem::findChildren(std::string compartmentName)
{
    std::vector<std::string> childrenNames;

    for(std::map<std::string,CAPCompartment>::iterator comp = this->membraneStructure.begin();
    comp != this->membraneStructure.end();
    ++comp)
    {
        if((comp->second).getFather() == compartmentName)
        {
            childrenNames.push_back((comp->second).getName());
        }
    }

    return childrenNames;
}

void PSystem::assignChildren()
{
    for(std::map<std::string,CAPCompartment>::iterator comp = this->membraneStructure.begin();
    comp != this->membraneStructure.end();
    ++comp)
    {
        (comp->second).setChildren(findChildren((comp->second).getName()));
    }
}

void PSystem::createInitialMultisets(xmlpp::Node::NodeList initialMultisetsDeclaration)
{

  for(xmlpp::Node::NodeList::iterator ini = initialMultisetsDeclaration.begin();
      ini != initialMultisetsDeclaration.end();
      ++ini)
    {
      InitialMultiset i(*ini);
      (this->initialMultisets).insert(pair<std::string,InitialMultiset>(i.getLabel(),i));
    }
}

void PSystem::createRuleSets(xmlpp::Node::NodeList ruleSetsDeclaration)
{

  for(xmlpp::Node::NodeList::iterator rul = ruleSetsDeclaration.begin();
      rul != ruleSetsDeclaration.end();
      ++rul)
    {
      CAPRuleSet r(*rul);
      (this->ruleSets).insert(pair<std::string,CAPRuleSet>(r.getLabel(),r));
    }
}


void PSystem::checkAlphabet()
{
    std::list<std::string> ruleObjects = getRuleObjects();
    std::list<std::string> alphabetObjects((this->alphabet).begin(),(this->alphabet).end());
    alphabetObjects.sort();
    alphabetObjects.unique();

    if(ruleObjects != alphabetObjects)
    {
        std::cout << "Warning missing objects in the alphabet from P system " << this->name << std::endl;
        std::cout << "Suggested Alphabet:" << std::endl;
        std::vector<std::string> newAlphabet;
        for(std::list<std::string>::iterator o = ruleObjects.begin();
            o != ruleObjects.end();
            ++o)
        {
            std::cout << "<object name=\"" << *o << "\" />" << std::endl;
            newAlphabet.push_back(*o);
        }

        std::cout << "Do you want to change the alphabet? (y/n): ";
        std::string answer;
        std::cin >> answer;
        if(answer == "y")
        {
            this->alphabet = newAlphabet;
        }
        else
        {
            exit(EXIT_FAILURE);
        }
    }

}

//Constructor from sbml specification
PSystem::PSystem(SBMLDocument* sbmlDocument) //std::string fileName)
{

    //Checking for errors
    unsigned int numErrors = sbmlDocument->getNumErrors();
    if(numErrors > 0)
    {
        cerr << "Errors when reading sbml file" << std::endl;
	sbmlDocument->printErrors(cerr);
	exit(EXIT_FAILURE);
    }


    if(sbmlDocument->getLevel() == 1)
    {
        cerr << "Error when reading sbml file: incompatible version" << std::endl;
	exit(EXIT_FAILURE);
    }

    Model* sbmlModel = sbmlDocument->getModel();

    //Set model name
    this->name = sbmlModel->getId();

    //Construct the different components
    createAlphabet(sbmlModel);
    createMembraneStructure(sbmlModel);
    assignChildren();
    createSetOfLabels();
    createInitialMultisets(sbmlModel);
    createRuleSets(sbmlModel);

    /*createSetOfLabels(labelsDeclaration);
    createRuleSets(ruleSetsDeclaration);*/

}

void PSystem::createAlphabet(Model* sbmlModel)
{
    std::list<std::string> speciesNames;
    unsigned int numSpecies = sbmlModel->getNumSpecies();

    for(unsigned int s = 0; s < numSpecies; ++s)
    {
        std::string speciesName = sbmlModel->getSpecies(s)->getName();
        if(!isDegradation(speciesName))
        {
            speciesNames.push_front(speciesName);
        }
    }

    speciesNames.sort();
    speciesNames.unique();

    std::vector<std::string> species(speciesNames.begin(),speciesNames.end());

    this->alphabet = species;
}

// test if species name is degradation species
bool PSystem::isDegradation(std::string s)
{
	if(s == "s0" || s.find("_degraded") != string::npos)
		return true;
	return false;
}


void PSystem::createMembraneStructure(Model* sbmlModel)
{
    unsigned int numCompartments = sbmlModel->getNumCompartments();

    //std::cout << "number of compartments: " << numCompartments << std::endl;

    unsigned int numberId = 0;


    for(unsigned int c = 0; c < numCompartments; ++c)
    {
        CAPCompartment compartment;
        Compartment* sbmlCompartment = sbmlModel->getCompartment(c);

        if(sbmlCompartment->getId() != "default")
        {
            compartment.setName(sbmlCompartment->getName());
            compartment.setLabel(sbmlCompartment->getName());
            compartment.setNumberId(numberId);
            ++numberId;
            std::string fatherId = sbmlCompartment->getOutside();
            Compartment* fatherCompartment = sbmlModel->getCompartment(fatherId);
            compartment.setFather(fatherCompartment->getName());
            (this->membraneStructure).insert(pair<std::string,CAPCompartment>(compartment.getName(),compartment));
        }
    }

}

void PSystem::createSetOfLabels()
{
    for(std::map<std::string,CAPCompartment>::iterator comp = this->membraneStructure.begin();
    comp != this->membraneStructure.end();
    ++comp)
    {
        this->setOfLabels.push_back(comp->first);
    }
}

void PSystem::createInitialMultisets(Model* sbmlModel)
{
    unsigned int numSpecies = sbmlModel->getNumSpecies();

    for(unsigned int s = 0; s < numSpecies; ++s)
    {
        std::string speciesName = sbmlModel->getSpecies(s)->getName();
        unsigned int multiplicity = sbmlModel->getSpecies(s)->getInitialAmount();
        if(multiplicity > 0)
        {
            std::string compartmentId = sbmlModel->getSpecies(s)->getCompartment();
            std::string compartmentName = sbmlModel->getCompartment(compartmentId)->getName();
            std::map<std::string,InitialMultiset>::iterator it = this->initialMultisets.find(compartmentName);
            if(it == this->initialMultisets.end())
            {
                InitialMultiset iniMultiset;
                iniMultiset.setLabel(compartmentName);
                iniMultiset.setObjectMultiplicity(speciesName,multiplicity);
                this->initialMultisets.insert(pair<std::string,InitialMultiset>(compartmentName,iniMultiset));
            }
            else
            {
                (it->second).setObjectMultiplicity(speciesName,multiplicity);
            }
        }
    }

}

void PSystem::createRuleSets(Model* sbmlModel)
{
    unsigned int numReactions = sbmlModel->getNumReactions();

    for(unsigned int r = 0; r < numReactions; ++r)
    {
        unsigned int numReactants = sbmlModel->getReaction(r)->getNumReactants();
        Reaction* reaction = sbmlModel->getReaction(r);

        checkValid(sbmlModel,reaction);

        //Create a rule from the corresponding reaction
        CAPRule rule;

        rule.setName(reaction->getId());

        //The rule is associated with the compartment where the reactants are located
        std::string compartmentId = sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment();
        std::string compartmentName = sbmlModel->getCompartment(compartmentId)->getName();

        //The constant associated with the rule is the paramter associated with the kinetic law
        if(!reaction->isSetKineticLaw())
        {
            cerr << "No constant associated with reaction " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }
        else if(reaction->getKineticLaw()->getNumParameters() == 0)
        {
            cerr << "No constant associated with reaction " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }
        else if(reaction->getKineticLaw()->getNumParameters() > 1)
        {
            cerr << "More than a constant associated with reaction " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }

        rule.setStringConstant((reaction->getKineticLaw())->getParameter(0)->getName());
        rule.setConstant((reaction->getKineticLaw())->getParameter(0)->getValue());

        //Getting the reactants and products of the reaction to specify the lhs and rhs of the rule
        std::vector<std::string> lhsObjects = reactants(sbmlModel, reaction);
        std::vector<std::string> rhsObjects = products(sbmlModel, reaction);

        //Depending on the type of reaction the rule is created accordingly
        std::string type = reactionType(sbmlModel, reaction);
        if(type == "transformation")
        {
            rule.setType("boundary");
            rule.setLhsInside(lhsObjects);
            rule.setRhsInside(rhsObjects);
            rule.setLabel(sbmlModel->getCompartment(sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment())->getName());
            addRule(compartmentName,rule);
        }
        else if(type == "diffusionOut")
        {
            rule.setType("boundary");
            rule.setLhsInside(lhsObjects);
            rule.setRhsOutside(rhsObjects);
            rule.setLabel(sbmlModel->getCompartment(sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment())->getName());
            addRule(compartmentName,rule);
        }
        else if(type == "diffusionIn")
        {
            rule.setType("boundary");
            rule.setLhsOutside(lhsObjects);
            rule.setRhsInside(rhsObjects);
            rule.setLabel(sbmlModel->getCompartment(sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment())->getName());
            addRule(compartmentName,rule);
        }
        else if(type == "translocation")
        {
            //The diffusion out of the system is represented as a translocation rule sending objects to the neighbours of the P system
            //in the lattice. We assume a rectangular lattice with only four neighbours.
            rule.setType("translocation");
            rule.setLhsTranslocation(lhsObjects);
            rule.setRhsTranslocation(rhsObjects);
            rule.setLabel(sbmlModel->getCompartment(sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment())->getName());

            rule.setLabelLhs(sbmlModel->getCompartment(sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment())->getName());

            //We add four rules so the constant rule of each new rule need to be reajusted in order not to disturb the dynamics of the system.
            double ruleConstant = rule.getConstant();
            ruleConstant = ruleConstant / 4;
            rule.setConstant(ruleConstant);

            std::vector<int> ruleVector(3,0);

            CAPRule ruleW = rule;
            ruleVector[0] = -1;
            ruleVector[1] = 0;
            ruleW.setRuleVector(ruleVector);
            addRule(compartmentName,ruleW);

            CAPRule ruleE = rule;
            ruleVector[0] = 1;
            ruleVector[1] = 0;
            ruleE.setRuleVector(ruleVector);
            addRule(compartmentName,ruleE);

            CAPRule ruleN = rule;
            ruleVector[0] = 0;
            ruleVector[1] = 1;
            ruleN.setRuleVector(ruleVector);
            addRule(compartmentName,ruleN);

            CAPRule ruleS = rule;
            ruleVector[0] = 0;
            ruleVector[1] = -1;
            ruleS.setRuleVector(ruleVector);
            addRule(compartmentName,ruleS);

        }
    }

}

void PSystem::checkValid(Model* sbmlModel, Reaction* reaction)
{
    unsigned int numReactants = reaction->getNumReactants();
    unsigned int numProducts = reaction->getNumProducts();

    //We only support reactions with at least one reactant and one product and no more than two reactants and two products.
    if(numReactants == 0 | numReactants > 2)
    {
        cerr << "Invalid number of reactants:" << numReactants << " in rule " << reaction->getId() << std::endl;
        exit(EXIT_FAILURE);
    }

    if(numProducts == 0 | numProducts > 2)
    {
        cerr << "Invalid number of products: " << numProducts << " in rule " << reaction->getId() << std::endl;
        exit(EXIT_FAILURE);
    }

    //All the reactants and products must be in the same compartment if we have either two reactants or two products
    if(numReactants == 2 && numProducts == 1)
    {
        std::string compartmentName1 = sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment();
        std::string compartmentName2 = sbmlModel->getSpecies(reaction->getReactant(1)->getSpecies())->getCompartment();
        std::string compartmentName3 = sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment();
        if(compartmentName1 != compartmentName2 | compartmentName1 != compartmentName3)
        {
            cerr << "Reactants and products in different compartments in rule " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }
    }

    if(numReactants == 2 && numProducts == 2)
    {
        std::string compartmentName1 = sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment();
        std::string compartmentName2 = sbmlModel->getSpecies(reaction->getReactant(1)->getSpecies())->getCompartment();
        std::string compartmentName3 = sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment();
        std::string compartmentName4 = sbmlModel->getSpecies(reaction->getProduct(1)->getSpecies())->getCompartment();
        if(compartmentName1 != compartmentName2 | compartmentName1 != compartmentName3 | compartmentName1 != compartmentName4)
        {
            cerr << "Reactants and products in different compartments in rule " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }
    }

    if(numReactants == 1 && numProducts == 2)
    {
        std::string compartmentName1 = sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment();
        std::string compartmentName2 = sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment();
        std::string compartmentName3 = sbmlModel->getSpecies(reaction->getProduct(1)->getSpecies())->getCompartment();
        if(compartmentName1 != compartmentName2 | compartmentName1 != compartmentName3)
        {
            cerr << "Reactants and products in different compartments in rule " << reaction->getId() << std::endl;
            exit(EXIT_FAILURE);
        }
    }
}


std::string PSystem::reactionType(Model* sbmlModel, Reaction* reaction)
{

    std::string type;

    unsigned int numReactants = reaction->getNumReactants();
    unsigned int numProducts = reaction->getNumProducts();

    if(numReactants == 2 | numProducts == 2)
    {
        type = "transformation";
    }

    if(numReactants == 1 && numProducts == 1)
    {
        std::string reactantCompartment = sbmlModel->getSpecies(reaction->getReactant(0)->getSpecies())->getCompartment();
        std::string reactantOutside = sbmlModel->getCompartment(reactantCompartment)->getOutside();
        std::string productCompartment = sbmlModel->getSpecies(reaction->getProduct(0)->getSpecies())->getCompartment();
        std::string productOutside = sbmlModel->getCompartment(productCompartment)->getOutside();

        if(reactantOutside == productCompartment)
        {
            if(reactantOutside == "default")
            {
                type = "translocation";
            }
            else
            {
                type = "diffusionOut";
            }
        }
        else if(reactantCompartment == productOutside)
        {
            type = "diffusionIn";
        }
        else if(reactantCompartment == productCompartment)
        {
            type = "transformation";
        }
    }

    return type;

}

std::vector<std::string> PSystem::reactants(Model* sbmlModel, Reaction* reaction)
{
    std::vector<std::string> reactantNames;
    unsigned int numReactants = reaction->getNumReactants();

    for(unsigned int s = 0; s < numReactants; ++s)
    {
        std::string speciesId = reaction->getReactant(s)->getSpecies();
        std::string speciesName = sbmlModel->getSpecies(speciesId)->getName();
        reactantNames.push_back(speciesName);
    }

    return reactantNames;
}

std::vector<std::string> PSystem::products(Model* sbmlModel, Reaction* reaction)
{
    std::vector<std::string> productNames;
    unsigned int numProducts = reaction->getNumProducts();

    for(unsigned int s = 0; s < numProducts; ++s)
    {
        std::string speciesId = reaction->getProduct(s)->getSpecies();
        std::string speciesName = sbmlModel->getSpecies(speciesId)->getName();
        if(!isDegradation(speciesName))
        {
            productNames.push_back(speciesName);
        }
    }

    return productNames;
}

void PSystem::addRule(std::string compartmentName, CAPRule rule)
{
    std::map<std::string,CAPRuleSet>::iterator it = this->ruleSets.find(compartmentName);

    if(it == this->ruleSets.end())
    {
        CAPRuleSet ruleSet;
        ruleSet.setLabel(compartmentName);
        ruleSet.addRule(rule);
        this->ruleSets.insert(pair<std::string,CAPRuleSet>(ruleSet.getLabel(),ruleSet));
    }
    else
    {
        (it->second).addRule(rule);
    }
}

void PSystem::convertTranslocation()
{


    for(std::map<std::string,CAPRuleSet>::iterator rSet = this->ruleSets.begin();
        rSet != this->ruleSets.end();
        ++rSet)
    {
        std::vector<CAPRule> convertedRules;
        std::vector<CAPRule> rules = (rSet->second).getRules();

        for(std::vector<CAPRule>::iterator rule = rules.begin();
        rule != rules.end();
        ++rule)
        {
            convertedRules.push_back(convertTranslocationRule(*rule, rSet->first));
        }


        (rSet->second).setRules(convertedRules);

    }

}


CAPRule PSystem::convertTranslocationRule(CAPRule rule, std::string compartmentName)
{
    CAPRule convertedRule;

    if((rule.getLhsOutside().empty() && rule.getRhsOutside().empty()) | (rule.getType() == "translocation"))
    {
        convertedRule = rule;
    }
    //[ A ]_name -> A [ ]_name
    else if(rule.getLhsOutside().empty() && !rule.getLhsInside().empty() && !rule.getRhsOutside().empty() && rule.getRhsInside().empty())
    {
        convertedRule.setConstant(rule.getConstant());
        convertedRule.setLabel(rule.getLabel());
        convertedRule.setLabelLhs(rule.getLabel());
        convertedRule.setName(rule.getName());
        convertedRule.setStringConstant(rule.getStringConstant());
        convertedRule.setType("translocation");

        convertedRule.setLhsTranslocation(rule.getLhsInside());
        convertedRule.setRhsTranslocation(rule.getRhsOutside());

        //Determination of the rule vector
        int sourceId = (this->membraneStructure)[compartmentName].getNumberId();
        std::string fatherCompartment = (this->membraneStructure)[compartmentName].getFather();
        int targetId = (this->membraneStructure)[fatherCompartment].getNumberId();
        std::vector<int> ruleVector(2,0);
        ruleVector.push_back(targetId - sourceId);

        convertedRule.setRuleVector(ruleVector);
    }
    //A [ ]_name -> [ A ]_name
    else if(!rule.getLhsOutside().empty() && rule.getLhsInside().empty() && rule.getRhsOutside().empty() && !rule.getRhsInside().empty())
    {
        convertedRule.setConstant(rule.getConstant());
        convertedRule.setLabel(rule.getLabel());
        convertedRule.setLabelLhs(rule.getLabel());
        convertedRule.setName(rule.getName());
        convertedRule.setStringConstant(rule.getStringConstant());
        convertedRule.setType("translocation");

        convertedRule.setLhsTranslocation(rule.getLhsOutside());
        convertedRule.setRhsTranslocation(rule.getRhsInside());
        unsigned long sourceId = (this->membraneStructure)[compartmentName].getNumberId();
        unsigned long targetId = (this->membraneStructure)[rule.getLabel()].getNumberId();
        std::vector<int> ruleVector(2,0);
        ruleVector.push_back(targetId - sourceId);
        convertedRule.setRuleVector(ruleVector);
    }
    else
    {
        cerr << "Rule type no suported: " << std::endl;
        rule.showRule();
        exit(EXIT_FAILURE);
    }

    return convertedRule;
}

void PSystem::show()
{
  std::cout << "-----------------------------" << std::endl;
  std::cout << "P System: " << this->name << std::endl;
  std::cout << "-----------------------------" << std::endl;

  //Show the alphabet
  std::cout << "Alphabet: " << std::endl;
  for(std::vector<std::string>::iterator obj = this->alphabet.begin();
      obj != this->alphabet.end();
      ++obj)
    {
      std::cout << "\t" << *obj << std::endl;
    }

  //Show the labels
  std::cout << "Labels: " << std::endl;
  for(std::vector<std::string>::iterator lab = this->setOfLabels.begin();
      lab != this->setOfLabels.end();
      ++lab)
    {
      std::cout << "\t" << *lab << std::endl;
    }

  //Show membrane structure
  std::cout << "Membrane Structure:" << std::endl;
  for(std::map<std::string,CAPCompartment>::iterator comp = this->membraneStructure.begin();
      comp != this->membraneStructure.end();
      ++comp)
    {
      (comp->second).show();
    }
  //Show initial multisets
  std::cout << "Initial Multisets: " << std::endl;
  for(std::map<std::string,InitialMultiset>::iterator ini = this->initialMultisets.begin();
      ini != this->initialMultisets.end();
      ++ini)
    {
      (ini->second).show();
    }
  //Show set of rules
  std::cout << "Set of Rules " << std::endl;
  for(std::map<std::string,CAPRuleSet>::iterator rSet = ruleSets.begin();
      rSet != ruleSets.end();
      ++rSet)
    {
      (rSet->second).showRuleSet();
    }
}

//Getters
std::string PSystem::getName() { return this->name; }

std::vector<std::string> PSystem::getAlphabet() { return this->alphabet; }

std::map<std::string,CAPRuleSet> PSystem::getRuleSets()
{
  return this->ruleSets;
}

std::map<std::string,InitialMultiset> PSystem::getInitialMultisets()
{
  return this->initialMultisets;
}

InitialMultiset PSystem::getInitialMultiset(std::string label)
{
    return (this->initialMultisets)[label];
}

unsigned long PSystem::getNumberOfCompartments()
{
  return (this->membraneStructure).size();
}

std::map<std::string,CAPCompartment> PSystem::getMembraneStructure()
{
  return this->membraneStructure;
}

std::list<std::string> PSystem::getRuleObjects()
{
    std::list<std::string> objects;

    for(std::map<std::string,CAPRuleSet>::iterator rSet = (this->ruleSets).begin();
    rSet != (this->ruleSets).end();
    ++rSet)
    {
        std::list<std::string> ruleSetObjects = (rSet->second).getObjects();

        std::list<std::string>::iterator it = objects.begin();
        objects.splice(it,ruleSetObjects);
    }

    objects.sort();
    objects.unique();

    return objects;

}


void PSystem::setName(std::string pSystemName)
{
    this->name = pSystemName;
}

void PSystem::setObjectMultiplicity(std::string label, std::string objectName, unsigned long objectMultiplicity)
{
    (this->initialMultisets)[label].setObjectMultiplicity(objectName,objectMultiplicity);
}

////////////////////////////
// Rule class definitions //
////////////////////////////

CAPRule::CAPRule()
{

}

CAPRule::~CAPRule()
{

}

//Copy constructors
CAPRule::CAPRule(const CAPRule& capRule)
{
    this->constant = capRule.constant;
    this->label = capRule.label;
    this->labelLhs = capRule.labelLhs;
    this->lhsInside = capRule.lhsInside;
    this->lhsOutside = capRule.lhsOutside;
    this->lhsTranslocation = capRule.lhsTranslocation;
    this->name = capRule.name;
    this->rhsInside = capRule.rhsInside;
    this->rhsOutside = capRule.rhsOutside;
    this->rhsTranslocation = capRule.rhsTranslocation;
    this->ruleVector = capRule.ruleVector;
    this->stringConstant = capRule.stringConstant;
    this->type = capRule.type;
}


CAPRule::CAPRule(xmlpp::Node* ruleDeclaration){

  //We cast rule_node to Element to get its attributes
  xmlpp::Element* rule = dynamic_cast<xmlpp::Element*>(ruleDeclaration);

  //Getting the name of the rule
  this->name = (rule->get_attribute("name"))->get_value();
  //Getting the type of the rule
  xmlpp::Attribute* typeDeclaration = rule->get_attribute("type");
  if(typeDeclaration != NULL)
  {
      this->type = typeDeclaration->get_value();
  }

  //Getting the vector associated with the rule if it is a translocation rule
  xmlpp::Attribute* vectorDeclaration = rule->get_attribute("vector");
  if(vectorDeclaration != NULL)
  {
      std::string vectorString = vectorDeclaration->get_value();
      std::string::size_type commaPosition = vectorString.find(",", 0);
      std::string xString = vectorString.substr(0,commaPosition);
      std::string yString = vectorString.substr(commaPosition+1);
      this->ruleVector.push_back(fromStringTo<int>(xString));
      this->ruleVector.push_back(fromStringTo<int>(yString));
      this->ruleVector.push_back(0);
  }

  //Getting the constant associated with the rule distinguishing between an abstract
  //rule defined in a module and an actual rule
  xmlpp::Attribute* ruleConstant = rule->get_attribute("constant");
  if(ruleConstant == NULL)
  {
      cerr << "Error: rule " << this->name << " without constant" << std::endl;
      exit(EXIT_FAILURE);
  }
  else
  {
      this->stringConstant = ruleConstant->get_value();
  }

  xmlpp::Attribute* constantValue = rule->get_attribute("value");
  if(constantValue == NULL)
  {
      this->constant = 0;
  }
  else
  {
      this->constant = fromStringTo<double>(constantValue->get_value());
  }

  //Getting the LHS
  xmlpp::Node* lhs = (ruleDeclaration->get_children("lhs")).front();

  //Getting the outside if it is not empty
  xmlpp::Node::NodeList listOfOutsideObjects = lhs->get_children("listOfOutsideObjects");
  if(!listOfOutsideObjects.empty())
  {
      xmlpp::Node* nodeOutsideObjects = listOfOutsideObjects.front();
      xmlpp::Node::NodeList outsideObjects = nodeOutsideObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = outsideObjects.begin();
	  obj != outsideObjects.end();
	  ++obj)
      {
          xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
          std::string objectName = (object->get_attribute("name"))->get_value();
          this->lhsOutside.push_back(objectName);
     }
  }

  //Getting the inside if it is not empty
  xmlpp::Node::NodeList listOfInsideObjects = lhs->get_children("listOfInsideObjects");
  if(!listOfInsideObjects.empty())
  {
      xmlpp::Node* nodeInsideObjects = listOfInsideObjects.front();
      xmlpp::Node::NodeList insideObjects = nodeInsideObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = insideObjects.begin();
	  obj != insideObjects.end();
	  ++obj)
      {
          xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
          std::string objectName = (object->get_attribute("name"))->get_value();
          this->lhsInside.push_back(objectName);
     }

      //Get label of the lhs inside objects
      xmlpp::Element* objectsInsideElement = dynamic_cast<xmlpp::Element*>(nodeInsideObjects);
      this->label = (objectsInsideElement->get_attribute("label"))->get_value();
  }

  //Getting the translocated objects if not empty
  xmlpp::Node::NodeList listOfTranslocatedObjects = lhs->get_children("listOfTranslocatedObjects");
  if(!listOfTranslocatedObjects.empty())
  {
      xmlpp::Node* nodeTranslocatedObjects = listOfTranslocatedObjects.front();
      xmlpp::Node::NodeList translocatedObjects = nodeTranslocatedObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = translocatedObjects.begin();
	  obj != translocatedObjects.end();
	  ++obj)
	{
	  xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
	  std::string objectName = (object->get_attribute("name"))->get_value();
	  lhsTranslocation.push_back(objectName);
	}
      //Get label of the lhs translocated objects
      xmlpp::Element* translocatedElement = dynamic_cast<xmlpp::Element*>(nodeTranslocatedObjects);
      this->labelLhs = (translocatedElement->get_attribute("label"))->get_value();
  }

  //Getting the RHS
  xmlpp::Node* rhs = (ruleDeclaration->get_children("rhs")).front();

//Getting the outside if it is not empty
  listOfOutsideObjects = rhs->get_children("listOfOutsideObjects");
  if(!listOfOutsideObjects.empty())
  {
      xmlpp::Node* nodeOutsideObjects = listOfOutsideObjects.front();
      xmlpp::Node::NodeList outsideObjects = nodeOutsideObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = outsideObjects.begin();
	  obj != outsideObjects.end();
	  ++obj)
      {
          xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
          std::string objectName = (object->get_attribute("name"))->get_value();
          this->rhsOutside.push_back(objectName);
     }
  }

  //Getting the inside if it is not empty
  listOfInsideObjects = rhs->get_children("listOfInsideObjects");
  if(!listOfInsideObjects.empty())
  {
      xmlpp::Node* nodeInsideObjects = listOfInsideObjects.front();
      xmlpp::Node::NodeList insideObjects = nodeInsideObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = insideObjects.begin();
	  obj != insideObjects.end();
	  ++obj)
      {
          xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
          std::string objectName = (object->get_attribute("name"))->get_value();
          this->rhsInside.push_back(objectName);
     }

     //Get label of the lhs inside objects
      xmlpp::Element* objectsInsideElement = dynamic_cast<xmlpp::Element*>(nodeInsideObjects);
      this->label = (objectsInsideElement->get_attribute("label"))->get_value();
  }

  //Getting the translocated objects if not empty
  listOfTranslocatedObjects = rhs->get_children("listOfTranslocatedObjects");
  if(!listOfTranslocatedObjects.empty())
  {
      xmlpp::Node* nodeTranslocatedObjects = listOfTranslocatedObjects.front();
      xmlpp::Node::NodeList translocatedObjects = nodeTranslocatedObjects->get_children("object");
      for(xmlpp::Node::NodeList::iterator obj = translocatedObjects.begin();
	  obj != translocatedObjects.end();
	  ++obj)
	{
	  xmlpp::Element* object = dynamic_cast<xmlpp::Element*>(*obj);
	  std::string objectName = (object->get_attribute("name"))->get_value();
	  rhsTranslocation.push_back(objectName);
	}
      //Get label of the lhs translocated objects
      xmlpp::Element* translocatedElement = dynamic_cast<xmlpp::Element*>(nodeTranslocatedObjects);
      this->label = (translocatedElement->get_attribute("label"))->get_value();
  }

  //Determine the type of rule depending on the objects in the rule
  if(!lhsTranslocation.empty())
  {
      this->type = "translocation";
  }
  else
  {
      this->type = "boundary";
  }
}


void CAPRule::showRule()
{

  if(this->type == "boundary")
    {
      // We show the objects outside on the left hand side
      for(std::vector<std::string>::iterator obj = lhsOutside.begin();
	  obj != lhsOutside.end();
	  ++obj)
	{
	  std::cout << *obj << " " ;
	}

      std::cout << "[ ";

      // We show the objects inside on the left hand side
      for(std::vector<std::string>::iterator obj = lhsInside.begin();
	  obj != lhsInside.end();
	  ++obj)
	{
	  std::cout << *obj << " " ;
	}

      std::cout << "]_" << label << " --> ";

      // We show the objects outside on the right hand side
      for(std::vector<std::string>::iterator obj = rhsOutside.begin();
	  obj != rhsOutside.end();
	  ++obj)
	{
	  std::cout << *obj << " " ;
	}

      std::cout << "[ ";

      // We show the objects inside on the right hand side
      for(std::vector<std::string>::iterator obj = rhsInside.begin();
	  obj != rhsInside.end();
	  ++obj)
	{
	  std::cout << *obj << " " ;
	}

      std::cout << "]_" << label << "        ";
    }
  else if(this->type == "translocation")
    {
      //We show the objects on the left hand side
      std::cout << "[ " ;

      for(std::vector<std::string>::iterator obj = lhsTranslocation.begin();
	  obj != lhsTranslocation.end();
	  ++obj)
	{
	  std::cout << *obj << " ";
	}
      std::cout << "]_" << this->labelLhs << "=(" << this->ruleVector[0] << "," << this->ruleVector[1] << "," << this->ruleVector[2] << ")=";
      std::cout << "[ ] --> [ ]_" << labelLhs << "=(" << this->ruleVector[0] << "," << this->ruleVector[1] << "," << this->ruleVector[2] << ")=";
      std::cout << "[ ";

      for(std::vector<std::string>::iterator obj = rhsTranslocation.begin();
	  obj != rhsTranslocation.end();
	  ++obj)
	{
	  std::cout << *obj << " ";
	}

      std::cout << "]" << "\t";
    }


  std::cout << this->stringConstant << " = " << this->constant << std::endl;

/*  if(this->constant == 0)
	{
	  std::cout << this->stringConstant << std::endl;
	}
      else
	{
	  std::cout << this->constant << std::endl;
	}
*/

}

std::list<std::string> CAPRule::getObjects()
{

  std::list<std::string> listLhsOutside(lhsOutside.begin(),lhsOutside.end());
  std::list<std::string> listLhsInside(lhsInside.begin(),lhsInside.end());
  std::list<std::string> listLhsTranslocation(lhsTranslocation.begin(),lhsTranslocation.end());
  std::list<std::string> listRhsOutside(rhsOutside.begin(),rhsOutside.end());
  std::list<std::string> listRhsInside(rhsInside.begin(),rhsInside.end());
  std::list<std::string> listRhsTranslocation(rhsTranslocation.begin(),rhsTranslocation.end());

  std::list<std::string> objects = listRhsInside;

  std::list<std::string>::iterator it = objects.begin();

  objects.splice(it,listRhsOutside);
  objects.splice(it,listRhsTranslocation);
  objects.splice(it,listLhsInside);
  objects.splice(it,listLhsOutside);
  objects.splice(it,listLhsTranslocation);
  objects.sort();
  objects.unique();

  return objects;

}


//Setters
void CAPRule::setName(std::string str)
{
    this->name = str;
}

void CAPRule::setType(std::string str)
{
    this->type = str;
}

void CAPRule::setConstant(double db)
{
   this->constant = db;
}

void CAPRule::setStringConstant(std::string str)
{
   this->stringConstant = str;
}

void CAPRule::setLhsOutside(std::vector<std::string> objects)
{
   this->lhsOutside = objects;
}

void CAPRule::setLhsInside(std::vector<std::string> objects)
{
   this->lhsInside = objects;
}

void CAPRule::setLhsTranslocation(std::vector<std::string> objects)
{
   this->lhsTranslocation = objects;
}

void CAPRule::setRhsOutside(std::vector<std::string> objects)
{
   this->rhsOutside = objects;
}

void CAPRule::setRhsInside(std::vector<std::string> objects)
{
   this->rhsInside = objects;
}

void CAPRule::setRhsTranslocation(std::vector<std::string> objects)
{
    this->rhsTranslocation = objects;
}

void CAPRule::setRuleVector(std::vector<int> coordinates)
{
    this->ruleVector = coordinates;
}

void CAPRule::setLabelLhs(std::string str)
{
    this->labelLhs = str;
}

void CAPRule::setLabel(std::string str)
{
    this->label = str;
}

void CAPRule::substituteObjects(map<std::string,std::string> substitution)
{

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = lhsOutside.begin();
	      o != lhsOutside.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
		  o->replace(pos,(it->first).length(),it->second);
		  pos = o->find(it->first,pos+1);
		}
	    }
	}

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = lhsInside.begin();
	      o != lhsInside.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
                  o->replace(pos,(it->first).length(),it->second);
		  pos = o->find(it->first,pos+1);
		}
	    }
	}

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = lhsTranslocation.begin();
	      o != lhsTranslocation.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
		  o->replace(pos,(it->first).length(),it->second);
		  pos = o->find(it->first,pos+1);
		}
	    }
	}

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = rhsOutside.begin();
	      o != rhsOutside.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
		  o->replace(pos,(it->first).length(),it->second);
		  pos = o->find(it->first,pos+1);
		}
	    }
	}

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = rhsInside.begin();
	      o != rhsInside.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
                  o->replace(pos,(it->first).length(),it->second);
                  pos = o->find(it->first,pos+1);
		}
	    }
	}

      for(map<std::string,std::string>::iterator it = substitution.begin();
	  it != substitution.end();
	  ++it)
	{
	  for(std::vector<std::string>::iterator o = rhsTranslocation.begin();
	      o != rhsTranslocation.end();
	      ++o)
	    {
	      size_t pos = o->find(it->first);
	      while(pos != o->npos)
		{
		  o->replace(pos,(it->first).length(),it->second);
		  pos = o->find(it->first,pos+1);
		}
	    }
	}

}
void CAPRule::substituteConstants(map<std::string,std::string> substitution)
{
    if(!substitution.empty() && this->constant == 0)
    {
        if(isNumeric(substitution[this->stringConstant]))
        {
            this->constant = fromStringTo<double>(substitution[this->stringConstant]);
        }
        else
        {
            this->stringConstant = substitution[this->stringConstant];
        }
    }
}

void CAPRule::substituteConstants(map<std::string,double> substitution)
{
    if(!substitution.empty() && this->constant == 0)
    {
        this->constant = substitution[this->stringConstant];
    }

  /*if(substitution.count(this->stringConstant) > 0)
    this->constant = substitution[this->stringConstant];
  else
    this->constant = fromStringTo<double>(this->stringConstant);
   */
}


void CAPRule::substituteLabels(map<std::string,std::string> substitution)
{
    if(!substitution.empty())
    {
      this->labelLhs = substitution[this->labelLhs];
      this->label = substitution[this->label];
    }
}

//////////////////////////////////
// CAPRuleSet class definitions //
//////////////////////////////////

CAPRuleSet::CAPRuleSet()
{
    this->numberRules = 0;
}

CAPRuleSet::CAPRuleSet(const CAPRuleSet& capRuleSet)
{
    this->label = capRuleSet.label;
    this->numberRules = capRuleSet.numberRules;
    this->setOfRules = capRuleSet.setOfRules;
}

CAPRuleSet::~CAPRuleSet()
{

}

CAPRuleSet::CAPRuleSet(xmlpp::Node* ruleSetDeclaration)
{
  //Get the label of the rule set declaration
  xmlpp::Element* ruleSet = dynamic_cast<xmlpp::Element*>(ruleSetDeclaration);
  this->label = (ruleSet->get_attribute("label"))->get_value();

  //Get all the nodes with the rules declarations
  xmlpp::Node::NodeList rules = ruleSetDeclaration->get_children("rule");

  for(Node::NodeList::iterator r = rules.begin(); r != rules.end(); ++r)
      setOfRules.push_back(CAPRule(*r));

  //Get all the nodes with the modules declarations
  xmlpp::Node::NodeList modules = ruleSetDeclaration->get_children("module");

  for(xmlpp::Node::NodeList::iterator m = modules.begin(); m != modules.end(); ++m)
    {
      std::vector<CAPRule> moduleRules = expand(*m);
      this->setOfRules.insert(this->setOfRules.end(),moduleRules.begin(),moduleRules.end());
    }

  //Remove the duplicates coming from different modules
  this->setOfRules = removeDuplicates(this->setOfRules);

  //Rename uniquely each rule
  this->setOfRules = individualRuleNames(this->setOfRules,this->label);

  //Get the number of rules in the rule set
  this->numberRules = this->setOfRules.size();

}

std::list<std::string> CAPRuleSet::getObjects()
{
    std::list<std::string> objects;

    for(std::vector<CAPRule>::iterator r = (this->setOfRules).begin();
        r != (this->setOfRules).end();
        ++r)
    {
        std::list<std::string> ruleObjects = r->getObjects();
        std::list<std::string>::iterator it = objects.begin();
        objects.splice(it,ruleObjects);
    }

    objects.sort();
    objects.unique();

    return objects;

}

//Setters
void CAPRuleSet::setLabel(std::string compartmentName)
{
    this->label = compartmentName;
}

void CAPRuleSet::setRules(std::vector<CAPRule> rules)
{
    this->setOfRules = rules;
    this->numberRules = rules.size();
}

void CAPRuleSet::addRule(CAPRule rule)
{
    this->setOfRules.push_back(rule);
    this->numberRules = this->numberRules + 1;
}


void CAPRuleSet::showRuleSet()
{
	std::cout << "Set of rules with label: " << this->label << std::endl << std::endl;
	std::cout << "Number of rules: " << numberRules << std::endl;
	std::cout << "Rules:" << std::endl;

	for(std::vector<CAPRule>::iterator rule = setOfRules.begin();
	    rule != setOfRules.end();
	    ++rule)
	  (*rule).showRule();
}

///////////////////////////////////////
// miscellaneos function definitions //
///////////////////////////////////////

std::vector<CAPRule> ECSB::expand(xmlpp::Node *moduleDeclaration)
{
    std::map<std::string,ModuleDefinition> partialLibrary;
    std::vector<CAPRule> rules = expand(moduleDeclaration,partialLibrary);
    return rules;
}


std::vector<CAPRule> ECSB::expand(xmlpp::Node *moduleDeclaration, std::map<std::string,ModuleDefinition> partialLibrary)
{
  //Information about the module and the library
  xmlpp::Element* module = dynamic_cast<xmlpp::Element*>(moduleDeclaration);
  std::string moduleName = (module->get_attribute("name"))->get_value();
  std::string file = (module->get_attribute("file"))->get_value();

  //Obtain the module definition from the corresponding library or partial library
  map<std::string,ModuleDefinition> modules;
  if(file == "this")
  {
      modules = partialLibrary;
  }
  else
  {
      ModuleLibrary moduleLibrary(file);
      modules = moduleLibrary.getModules();
  }

  if(modules.find(moduleName) == modules.end())
  {
      cerr << "Error: no module " << moduleName << " in library in file " << file << std::endl;
      exit(EXIT_FAILURE);
  }

  ModuleDefinition moduleDefinition = modules[moduleName];
  moduleDefinition = renameVariables(moduleDefinition);



  //Create substitutions vectors
  xmlpp::Attribute* objectsAttribute = module->get_attribute("objects");
  if(objectsAttribute == NULL)
  {
      cerr << "Error: no objects substitution for module " << moduleName << std::endl;
      exit(EXIT_FAILURE);
  }
  std::string objectList = objectsAttribute->get_value();
  map<std::string,std::string> objectSubstitution =
    createSubstitution(moduleDefinition.getObjects(),
		       convertToVector(objectList));

  /*std::cout << "Object substitution:" << std::endl;
  for(std::map<std::string,std::string>::iterator it = objectSubstitution.begin();
    it != objectSubstitution.end();
    ++it)
  {
      std::cout << it->first << "=" << it->second << std::endl;
  }
*/

  xmlpp::Attribute* constantsAttribute = module->get_attribute("constants");
  if(objectsAttribute == NULL)
  {
      cerr << "Error: no constants substitution for module " << moduleName << std::endl;
      exit(EXIT_FAILURE);
  }

  std::string constantList = constantsAttribute->get_value();
  map<std::string,std::string> constantSubstitution =
    createSubstitution(moduleDefinition.getConstants(),
		       convertToVector(constantList));

  /*std::cout << "String constant substitution:" << std::endl;
  for(std::map<std::string,std::string>::iterator it = constantSubstitution.begin();
    it != constantSubstitution.end();
    ++it)
  {
      std::cout << it->first << "=" << it->second << std::endl;
  }
  */

 /*
  std::map<std::string,double> constantSubstitutionDouble;
  for(map<std::string,std::string>::iterator m = constantSubstitutionStr.begin();
      m != constantSubstitutionStr.end(); ++m)
    {
      constantSubstitutionDouble.insert(pair<std::string,double>(m->first,
								 atof((m->second).c_str())));
    }

  std::cout << "Constant substitution:" << std::endl;
  for(std::map<std::string,double>::iterator it = constantSubstitutionDouble.begin();
    it != constantSubstitutionDouble.end();
    ++it)
  {
      std::cout << it->first << "=" << it->second << std::endl;
  }
*/

  xmlpp::Attribute* labelsAttribute = module->get_attribute("labels");
  if(labelsAttribute == NULL)
  {
      cerr << "Error: no labels substitution for module " << moduleName << std::endl;
      exit(EXIT_FAILURE);
  }

  std::string labelList = labelsAttribute->get_value();
  map<std::string,std::string> labelSubstitution =
    createSubstitution(moduleDefinition.getLabels(),
		       convertToVector(labelList));

/*  std::cout << "Label substitution:" << std::endl;
  for(std::map<std::string,std::string>::iterator it = labelSubstitution.begin();
    it != labelSubstitution.end();
    ++it)
  {
      std::cout << it->first << "=" << it->second << std::endl;
  }
*/
  std::vector<CAPRule> finalRules = moduleDefinition.instantiate(objectSubstitution, \
  		constantSubstitution, labelSubstitution);
  //return moduleDefinition.instantiate(objectSubstitution, \
  //		constantSubstitution, labelSubstitution);

  return finalRules;

}

//Auxiliar remove duplicate rules
std::vector<CAPRule> ECSB::removeDuplicates(std::vector<CAPRule> rules)
{
  std::vector<CAPRule> uniqueRules;
  int pos = 0;
  for(std::vector<CAPRule>::iterator r = rules.begin(); r != rules.end(); ++r)
    {
      pos++;
      if(ruleIn(*r,rules,pos)==0)
	uniqueRules.push_back(*r);
    }
 return uniqueRules;
}

//Auxiliar to associate unique names to rules after expansion and instantiation
std::vector<CAPRule> ECSB::individualRuleNames(std::vector<CAPRule> rules, std::string lab)
{
  unsigned long index = 1;

  for(std::vector<CAPRule>::iterator r = rules.begin(); r != rules.end(); ++r)
    {
      std::string ruleName = "r_" + lab + "_" + somethingToString<double>(index);
      r->setName(ruleName);
      index++;
    }
  return rules;
}

//Auxiliar function to determine whether or not a CAPRule is in a given vector of rules
//starting from a specific position in the vector specified by an iterator
int ECSB::ruleIn(CAPRule rule, std::vector<CAPRule> rules, int pos)
{
  int h = 0;
  for(std::vector<CAPRule>::iterator r = rules.begin(); r != rules.end(); ++r)
    {
      if( h >= pos )
	{
	  if(ruleEqual(rule,*r)==1)
	    return 1;
	}
      else
	{
	  h++;
	}
    }
  return 0;
}

int ECSB::ruleEqual(CAPRule r1, CAPRule r2)
{
  if(r1.getType() == r2.getType() &&
     r1.getConstant() == r2.getConstant() &&
     r1.getLabelLhs() == r2.getLabelLhs() &&
     r1.getLabel() == r2.getLabel() &&
     r1.getVect() == r2.getVect() &&
     equalityVector(r1.getLhsOutside(), r2.getLhsOutside()) &&
     equalityVector(r1.getLhsInside(), r2.getLhsInside()) &&
     equalityVector(r1.getLhsTranslocation(), r2.getLhsTranslocation()) &&
     equalityVector(r1.getRhsOutside(), r2.getRhsOutside()) &&
     equalityVector(r1.getRhsInside(), r2.getRhsInside()) &&
     equalityVector(r1.getRhsTranslocation(), r2.getRhsTranslocation())
     )
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

ModuleDefinition ECSB::renameVariables(ModuleDefinition module)
{
    std::vector<std::string> renameObjects;
    std::vector<std::string> objects = module.getObjects();

    for(std::vector<std::string>::iterator o = objects.begin();
        o != objects.end();
        ++o)
    {
        std::string renameObject = *o + "'";
        renameObjects.push_back(renameObject);
    }

    module.setObjects(renameObjects);

    std::map<std::string,std::string> objectSubstitution = createSubstitution(objects,renameObjects);
    std::map<std::string,std::string> constantSubstitution = createSubstitution(module.getConstants(),module.getConstants());
    std::map<std::string,std::string> labelSubstitution = createSubstitution(module.getLabels(),module.getLabels());

    std::vector<CAPRule> renameRules = module.instantiate(objectSubstitution, constantSubstitution, labelSubstitution);

    module.setRules(renameRules);

    return module;

}
